// API - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_config.c $
// $Rev: 23674 $
// $Date: 2013-10-31 16:25:34 -0500 (Thu, 31 Oct 2013) $

#include "dev_config.h"

#include "gsc_version.h"

#include "dev_access.h"
#include "dev_cable.h"
#include "dev_fifo.h"
#include "dev_irq.h"
#include "dev_misc.h"
#include "dev_rx.h"
#include "dev_tx.h"



/* typedefs	*****************************************************************/

typedef struct
{
	U32	parm;
	U32	(*func)(dev_data_t* dev, U32 which, UL set, UL* get);
} parm_item_t;

typedef struct
{
	U32					group;
	const parm_item_t*	list;
	U32					size;
} parm_group_t;

typedef struct
{
	const parm_group_t*	list;
	U32					size;
} parm_set_t;



/* variables	*************************************************************/

static const parm_item_t	_cable_list[]	=
{
	/* parm								func						*/
	{ HPDI32_CABLE_CLOCK_STATE,			cable_clock_state			},
	{ HPDI32_CABLE_COMMAND_MODE,		cable_command_mode			},
	{ HPDI32_CABLE_COMMAND_STATE,		cable_command_state			}
};

static const parm_item_t	_fifo_list[]	=
{
	/* parm							func				*/
	{ HPDI32_FIFO_ALMOST_LEVEL,		fifo_almost_level	},
	{ HPDI32_FIFO_RESET,			fifo_reset			},
	{ HPDI32_FIFO_SIZE,				fifo_size			},
	{ HPDI32_FIFO_STATUS,			fifo_status			},
	{ HPDI32_FIFO_TRANSFER_SIZE,	fifo_transfer_size	}
};

static const parm_item_t	_io_list[]	=
{
	/* parm							func					*/
	{ HPDI32_IO_ABORT,				io_abort				},
	{ HPDI32_IO_ABORTED,			io_aborted				},
	{ HPDI32_IO_BUFFER_POINTER,		io_buffer_pointer		},
	{ HPDI32_IO_BUFFER_SIZE,		io_buffer_size			},
	{ HPDI32_IO_CALLBACK_ARG,		io_callback_arg			},
	{ HPDI32_IO_CALLBACK_FUNC,		io_callback_func		},
	{ HPDI32_IO_DATA_SIZE,			io_data_size			},
	{ HPDI32_IO_DMA_CHANNEL_SEL,	io_dma_channel_select	},
	{ HPDI32_IO_DMA_CONTROL_MODE,	io_dma_control_mode		},
	{ HPDI32_IO_DMA_PRIORITY,		io_dma_priority			},
	{ HPDI32_IO_MODE,				io_mode					},
	{ HPDI32_IO_OVERLAP_ENABLE,		io_overlap_enable		},
	{ HPDI32_IO_PIO_THRESHOLD,		io_pio_threshold		},
	{ HPDI32_IO_STATUS,				io_status				},
	{ HPDI32_IO_TIMEOUT,			io_timeout				},
	{ HPDI32_IO_SINGLE_CYCLE,		io_single_cycle			}
};

static const parm_item_t	_irq_list[]	=
{
	/* parm							func					*/
	{ HPDI32_IRQ_CALLBACK_ARG,		irq_callback_arg		},
	{ HPDI32_IRQ_CALLBACK_FUNC,		irq_callback_func		},
	{ HPDI32_IRQ_ENABLE,			irq_enable				},
	{ HPDI32_IRQ_STATE,				irq_state				},
	{ HPDI32_IRQ_TRIGGER_CONFIG,	irq_trigger_config		}
};

static const parm_item_t	_misc_list[]	=
{
	/* parm							func					*/
	{ HPDI32_MISC_BOARD_JUMPERS,	misc_board_jumpers		},
	{ HPDI32_MISC_FAVOR_TX,			misc_favor_tx			},
	{ HPDI32_MISC_FEATURES,			misc_features			},
	{ HPDI32_MISC_MAP_GSC_REGS,		misc_map_gsc_regs		},
	{ HPDI32_MISC_MAP_GSC_REGS_PTR,	misc_map_gsc_regs_ptr	},
	{ HPDI32_MISC_MAP_PLX_REGS,		misc_map_plx_regs		},
	{ HPDI32_MISC_PCI_BUS_WIDTH,	misc_pci_bus_width		},
	{ HPDI32_MISC_STRICT_ARGUMENTS,	misc_strict_arguments	},
	{ HPDI32_MISC_STRICT_CONFIG,	misc_strict_config		},
	{ HPDI32_MISC_TX_RX_TRI_STATE,	misc_tx_rx_tri_state	}
};

static const parm_item_t	_rx_list[]	=
{
	/* parm						func			*/
	{ HPDI32_RX_ENABLE,			rx_enable		},
	{ HPDI32_RX_OVERRUN,		rx_overrun		},
	{ HPDI32_RX_ROW_COUNT,		rx_row_count	},
	{ HPDI32_RX_STATE,			rx_state		},
	{ HPDI32_RX_STATUS_COUNT,	rx_status_count	},
	{ HPDI32_RX_UNDER_RUN,		rx_under_run	}
};

static const parm_item_t	_tx_list[]	=
{
	/* parm								func						*/
	{ HPDI32_TX_AUTO_START,				tx_auto_start				},
	{ HPDI32_TX_AUTO_STOP,				tx_auto_stop				},
	{ HPDI32_TX_CLOCK_DIVIDER,			tx_clock_divider			},
	{ HPDI32_TX_ENABLE,					tx_enable					},
	{ HPDI32_TX_FLOW_CONTROL,			tx_flow_control				},
	{ HPDI32_TX_LINE_VALID_OFF_COUNT,	tx_line_valid_off_count		},
	{ HPDI32_TX_LINE_VALID_ON_COUNT,	tx_line_valid_on_count		},
	{ HPDI32_TX_OVERRUN,				tx_overrun					},
	{ HPDI32_TX_REMOTE_THROTTLE,		tx_remote_throttle			},
	{ HPDI32_TX_REMOTE_THROTTLE_STATE,	tx_remote_throttle_state	},
	{ HPDI32_TX_STATE,					tx_state					},
	{ HPDI32_TX_STATUS_VALID_COUNT,		tx_status_valid_count		},
	{ HPDI32_TX_STATUS_VALID_MIRROR,	tx_status_valid_mirror		}
};

static const parm_group_t	_groups[]	=
{
	/* group						list			size						*/
	{ HPDI32_CONFIG_GROUP_CABLE,	_cable_list,	ARRAY_ELEMENTS(_cable_list)	},
	{ HPDI32_CONFIG_GROUP_FIFO,		_fifo_list,		ARRAY_ELEMENTS(_fifo_list)	},
	{ HPDI32_CONFIG_GROUP_IO,		_io_list,		ARRAY_ELEMENTS(_io_list)	},
	{ HPDI32_CONFIG_GROUP_IRQ,		_irq_list,		ARRAY_ELEMENTS(_irq_list)	},
	{ HPDI32_CONFIG_GROUP_MISC,		_misc_list,		ARRAY_ELEMENTS(_misc_list)	},
	{ HPDI32_CONFIG_GROUP_RX,		_rx_list,		ARRAY_ELEMENTS(_rx_list)	},
	{ HPDI32_CONFIG_GROUP_TX,		_tx_list,		ARRAY_ELEMENTS(_tx_list)	}
};

static const parm_set_t	_parm_set	=
{
	/* list	*/	_groups,
	/* size	*/	ARRAY_ELEMENTS(_groups)
};



/*****************************************************************************
*
*	Function:	config_init
*
*	Purpose:
*
*		Verify that out data set is valid.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 config_init(void)
{
	unsigned int	i;
	unsigned int	j;
	U32				parm;
	U32				status	= (U32) GSC_SUCCESS;

	for (i = 0; i < _parm_set.size; i++)
	{
		for (j = 0; j < _parm_set.list[i].size; j++)
		{
			parm	= HPDI32_CONFIG_ENCODE(_parm_set.list[i].group, j);

			if (parm != _parm_set.list[i].list[j].parm)
			{
				status			= (U32) GSC_INIT_FAILURE;
				api_status_arg	= HPDI32_CONFIG_ENCODE(i, j);
			}
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	config_which_validate
*
*	Purpose:
*
*		Verify the given configuration "which" value against the given mask of
*		permissible bits.
*
*	Arguments:
*
*		dev			The device to act on.
*
*		which		The value to validate.
*
*		mask		The set of permissible bits.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 config_which_validate(dev_data_t* dev, U32 which, U32 mask)
{
	U32	status;

	if ((which & ~mask) == 0)
	{
		status	= (U32) GSC_SUCCESS;
	}
	else if (dev->strict_args)
	{
		status	= (U32) GSC_INVALID_DATA;
	}
	else
	{
		status	= (U32) GSC_SUCCESS;	// Ignore the extra bits.
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	hpdi32_config
*
*	Purpose:
*
*		Process a parameter configuration request per the given structure.
*
*	Arguments:
*
*		handle		The handle of the channel to close.
*
*		parm		The parameter to be acted upon.
*
*		which		A field of bits specifying the parameter specific
*					subcomponents to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT hpdi32_config(
	void*	handle,
	U32		parm,
	U32		which,
	UL		set,
	UL*		get)
{
	dev_data_t*			dev;
	const parm_group_t*	g_ptr	= NULL;
	U32					group;
	const parm_item_t*	i_ptr	= NULL;
	U32					index;
	U32					status	= (U32) GSC_SUCCESS;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		group	= HPDI32_CONFIG_GROUP_DECODE(parm);
		index	= HPDI32_CONFIG_INDEX_DECODE(parm);

		for (;;)	/* A convenience loop. */
		{
			if (group >= _parm_set.size)
			{
				status	= (U32) GSC_INVALID_DATA;
				break;
			}

			g_ptr	= _parm_set.list + group;

			if (index >= g_ptr->size)
			{
				status	= (U32) GSC_INVALID_DATA;
				break;
			}

			i_ptr	= g_ptr->list + index;

			if (i_ptr->parm != parm)
			{
				status	= (U32) GSC_INVALID_DATA;
				break;
			}

			if (i_ptr->func == NULL)
			{
				status	= (U32) GSC_UNSUPPORTED_FUNCTION;
				break;
			}

			status	= (i_ptr->func)(dev, which, set, get);
			break;
		}

		dev_access_unlock(dev);
	}

	return(status);
}


