// API - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_gpio.c $
// $Rev: 23674 $
// $Date: 2013-10-31 16:25:34 -0500 (Thu, 31 Oct 2013) $

#include "hpdi32_api.h"

#include "gsc_version.h"
#include "dev_access.h"
#include "dev_reg.h"



/* variables	*************************************************************/

static const struct
{
	U32	feature;
	U32	bcr_gpio;	// If a bit is set, then it is GPIO.
	U32	bcr_out;	// If this bit is set, then it is a GPIO output.
	U32 bcr_high;
	U32	bcr_low;
	U32	bsr;
	U8	value;
} _gpio_list[]	=
{
	{	/* GPIO 0	*/
		/* feature	*/	HPDI32_FSR_GPIO_0_5,
		/* bcr_gpio	*/	HPDI32_BCR_C1_GPIO,
		/* bcr_out	*/	HPDI32_BCR_C1_OUT,
		/* bcr_high	*/	HPDI32_BCR_C1_OUT_H,
		/* bcr_low	*/	HPDI32_BCR_C1_OUT_L,
		/* bsr		*/	HPDI32_BSR_D1,
		/* value	*/	0x01
	},
	{	/* GPIO 1	*/
		/* feature	*/	HPDI32_FSR_GPIO_0_5,
		/* bcr_gpio	*/	HPDI32_BCR_C2_GPIO,
		/* bcr_out	*/	HPDI32_BCR_C2_OUT,
		/* bcr_high	*/	HPDI32_BCR_C2_OUT_H,
		/* bcr_low	*/	HPDI32_BCR_C2_OUT_L,
		/* bsr		*/	HPDI32_BSR_D2,
		/* value	*/	0x02
	},
	{	/* GPIO 2	*/
		/* feature	*/	HPDI32_FSR_GPIO_0_5,
		/* bcr_gpio	*/	HPDI32_BCR_C3_GPIO,
		/* bcr_out	*/	HPDI32_BCR_C3_OUT,
		/* bcr_high	*/	HPDI32_BCR_C3_OUT_H,
		/* bcr_low	*/	HPDI32_BCR_C3_OUT_L,
		/* bsr		*/	HPDI32_BSR_D3,
		/* value	*/	0x04
	},
	{	/* GPIO 3	*/
		/* feature	*/	HPDI32_FSR_GPIO_0_5,
		/* bcr_gpio	*/	HPDI32_BCR_C4_GPIO,
		/* bcr_out	*/	HPDI32_BCR_C4_OUT,
		/* bcr_high	*/	HPDI32_BCR_C4_OUT_H,
		/* bcr_low	*/	HPDI32_BCR_C4_OUT_L,
		/* bsr		*/	HPDI32_BSR_D4,
		/* value	*/	0x08
	},
	{	/* GPIO 4	*/
		/* feature	*/	HPDI32_FSR_GPIO_0_5,
		/* bcr_gpio	*/	HPDI32_BCR_C5_GPIO,
		/* bcr_out	*/	HPDI32_BCR_C5_OUT,
		/* bcr_high	*/	HPDI32_BCR_C5_OUT_H,
		/* bcr_low	*/	HPDI32_BCR_C5_OUT_L,
		/* bsr		*/	HPDI32_BSR_D5,
		/* value	*/	0x10
	},
	{	/* GPIO 5	*/
		/* feature	*/	HPDI32_FSR_GPIO_0_5,
		/* bcr_gpio	*/	HPDI32_BCR_C6_GPIO,
		/* bcr_out	*/	HPDI32_BCR_C6_OUT,
		/* bcr_high	*/	HPDI32_BCR_C6_OUT_H,
		/* bcr_low	*/	HPDI32_BCR_C6_OUT_L,
		/* bsr		*/	HPDI32_BSR_D6,
		/* value	*/	0x20
	},
	{	/* GPIO 6	*/
		/* feature	*/	HPDI32_FSR_GPIO6_TXAS,
		/* bcr_gpio	*/	HPDI32_BCR_C0_GPIO,
		/* bcr_out	*/	HPDI32_BCR_C0_OUT,
		/* bcr_high	*/	HPDI32_BCR_C0_OUT_H,
		/* bcr_low	*/	HPDI32_BCR_C0_OUT_L,
		/* bsr		*/	HPDI32_BSR_D0,
		/* value	*/	0x40
	}
};




/*****************************************************************************
*
*	Function:	hpdi32_gpio_mod
*
*	Purpose:
*
*		Perform a read-modify-write operation on the GPIO port output value.
*		The value and mask is received with GPIO0 in the least significant bit
*		position.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		value		The value to apply.
*
*		mask		The set of bits to change.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT hpdi32_gpio_mod(void* handle, U8 value, U8 mask)
{
	U32			bcr;
	U8			bit;
	dev_data_t*	dev;
	int			i;
	U32			msk;
	U32			status;
	U32			val;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);

		if ((mask) && (status == (U32) GSC_SUCCESS))
		{
			msk	= 0;
			val	= 0;

			for (	i = 0, bit = 0x1;
					i < (int) ARRAY_ELEMENTS(_gpio_list);
					i++, bit *= 2)
			{
				if ((mask & bit) == 0)
					continue;	// Don't mod this bit.

				if ((dev->cache.fsr & _gpio_list[i].feature) == 0)
					continue;	// This GPIO feature is unavailable.

				if ((bcr & _gpio_list[i].bcr_gpio) == 0)
					continue;	// This bit is not GPIO.

				if ((bcr & _gpio_list[i].bcr_out) == 0)
					continue;	// This bit is an input.

				msk	|= _gpio_list[i].bcr_high | _gpio_list[i].bcr_low;

				if (value & bit)
					val	|= _gpio_list[i].bcr_high;
				else
					val	|= _gpio_list[i].bcr_low;
			}

			status	= GSC_REG_MOD(dev, HPDI32_BCR, val, msk);
		}

		dev_access_unlock(dev);
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	hpdi32_gpio_read
*
*	Purpose:
*
*		Read the current GPIO port value. The value is reported with GPIO0
*		in the least significant bit position. Only GPIO bits are reported.
*		All non-GPIO bits are reported as zero.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		value		The value read is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT hpdi32_gpio_read(void* handle, U8* value)
{
	U32			bcr;
	U32			bsr;
	dev_data_t*	dev;
	U8			gpio;
	int			i;
	U32			stat;
	U32			status;

	if (value == NULL)
	{
		status	= (U32) GSC_NULL_PARAM;
	}
	else
	{
		status	= dev_access_lock(handle, &dev);

		if (status == (U32) GSC_SUCCESS)
		{
			status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
			stat	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
			status	= (status == (U32) GSC_SUCCESS) ? stat : status;

			if (status == (U32) GSC_SUCCESS)
			{
				gpio	= 0;

				for (i = 0; i < (int) ARRAY_ELEMENTS(_gpio_list); i++)
				{
#ifdef _LIMIT_GPIO_READS_TO_GPIO_LINES
					if ((dev->cache.fsr & _gpio_list[i].feature) == 0)
						continue;	// This GPIO feature is unavailable.

					if ((bcr & _gpio_list[i].bcr_gpio) == 0)
						continue;	// This bit is not GPIO.
#endif

					if (bsr & _gpio_list[i].bsr)
						gpio	|= _gpio_list[i].value;
				}

				value[0]	= gpio;
			}

			dev_access_unlock(dev);
		}
	}

	return(status);
}



/*****************************************************************************
*
*	Function:	hpdi32_gpio_write
*
*	Purpose:
*
*		Update the current GPIO port output value. The value is received with
*		GPIO0 in the least significant bit position.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		value		The value to write.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
*****************************************************************************/

U32 GSC_EXPORT hpdi32_gpio_write(void* handle, U8 value)
{
	U32	status;

	status	= hpdi32_gpio_mod(handle, value, 0xFF);
	return(status);
}


