// API - Any Device - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/gsc_event.h $
// $Rev: 23674 $
// $Date: 2013-10-31 16:25:34 -0500 (Thu, 31 Oct 2013) $

#ifndef __GSC_EVENT_H__
#define __GSC_EVENT_H__

#include "gsc_os_event.h"
#include "gsc_os_thread.h"

#include "gsc_access.h"		/*lint !e537 */
#include "gsc_sem.h"		/*lint !e537 */



/* typedefs	*****************************************************************/

typedef struct
{
	/*
	*	All event structure must be allocated at run time so that the thread
	*	function can close down properly. The thread does the work of
	*	destroying and freeing the structure as it terminates. The event
	*	destroy service must be called to make this happen.
	*/

	gsc_sem_t		sem;	// Use when making changes or accessing content.
	gsc_device_t*	device;
	void*			id;
	os_event_t		event;	// Triggered when the associated event occurs.

	/*
	*	The below fields are used when an application wants to utilize the
	*	"callback" method of event notification.
	*/

	void			(*callback)(void* arg1, UL arg2, UL arg3);
	void*			arg1;
	UL				arg2;
	UL				arg3;
	os_thread_t		thread;		// Waits on the event and calls the callback function.
	U32				terminate;	// Set this to end the thread.
} gsc_event_t;



/* prototypes	*************************************************************/

U32		gsc_event_create(gsc_event_t** event, const char* name, gsc_device_t* device);
void	gsc_event_destroy(gsc_event_t** event);
void	gsc_event_trigger(gsc_event_t* event);
U32		gsc_event_wait(gsc_event_t* event, U32 timeout_ms);
U32		gsc_event_wait_2(gsc_event_t* e1, gsc_event_t* e2, U32 timeout_ms);

U32		gsc_event_config(	gsc_event_t*	event,
							void*			id,
							void			(*callback)(void* arg1, UL arg2, UL arg3),
							void*			arg1,
							UL				arg2,
							UL				arg3,
							gsc_event_t*	dest);



#endif
