// API - Any Device - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/gsc_io_pio.c $
// $Rev: 23674 $
// $Date: 2013-10-31 16:25:34 -0500 (Thu, 31 Oct 2013) $

#include "gsc_io.h"



/* #defines	*****************************************************************/

// Limit samples so we can ignore any time limit.
#define	_8_BIT_LIMIT_	(512L)
#define	_16_BIT_LIMIT_	(512L * 2)
#define	_32_BIT_LIMIT_	(512L * 4)



/******************************************************************************
*
*	Function:	gsc_io_pio_read_8_map
*
*	Purpose:
*
*		Read the specified number of bytes, assuming 8-bit samples. Perform
*		the read using memory mapped access to a firmware register.
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_read_8_map(gsc_io_t* io)
{
	U32					bytes;
	U8*					dest	= io->xfer_buf;
	volatile const U8*	src		= io->reg_ptr;

	if (io->xfer > _8_BIT_LIMIT_)
		bytes	= _8_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (; bytes--; dest++)
		dest[0]	= src[0];

	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_read_16_map
*
*	Purpose:
*
*		Read the specified number of bytes, assuming 16-bit samples. Perform
*		the read using memory mapped access to a firmware register.
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_read_16_map(gsc_io_t* io)
{
	U32					bytes;
	U16*				dest	= io->xfer_buf;
	volatile const U16*	src		= io->reg_ptr;

	if (io->xfer > _16_BIT_LIMIT_)
		bytes	= _16_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (bytes /= 2; bytes--; dest++)
		dest[0]	= src[0];

	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_read_32_map
*
*	Purpose:
*
*		Read the specified number of bytes, assuming 32-bit samples. Perform
*		the read using memory mapped access to a firmware register.
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_read_32_map(gsc_io_t* io)
{
	U32				bytes;
	U32*			dest	= io->xfer_buf;
	volatile U32*	src		= io->reg_ptr;

	if (io->xfer > _32_BIT_LIMIT_)
		bytes	= _32_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (bytes /= 4; bytes--; dest++)
		dest[0]	= src[0];

	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_write_8_map
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 8-bit samples. Perform
*		the write using memory mapped access to a firmware register.
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_write_8_map(gsc_io_t* io)
{
	U32				bytes;
	volatile U8*	dest;
	const U8*		src;

	dest	= io->reg_ptr;
	src		= io->xfer_buf;

	if (io->xfer > _8_BIT_LIMIT_)
		bytes	= _8_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (; bytes--; src++)
		dest[0]	= src[0];

	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_write_16_map
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 16-bit samples. Perform
*		the write using memory mapped access to a firmware register.
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_write_16_map(gsc_io_t* io)
{
	U32				bytes;
	volatile U16*	dest;
	const U16*		src;

	dest	= io->reg_ptr;
	src		= io->xfer_buf;

	if (io->xfer > _16_BIT_LIMIT_)
		bytes	= _16_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (bytes /= 2; bytes--; src++)
		dest[0]	= src[0];

	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_write_32_map
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 32-bit samples. Perform
*		the write using memory mapped access to a firmware register.
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_write_32_map(gsc_io_t* io)
{
	U32				bytes;
	volatile U32*	dest;
	const U32*		src;

	dest	= io->reg_ptr;
	src		= io->xfer_buf;

	if (io->xfer > _32_BIT_LIMIT_)
		bytes	= _32_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (bytes /= 4; bytes--; src++)
		dest[0]	= src[0];

	return((U32) GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_read_8_reg
*
*	Purpose:
*
*		Read the specified number of bytes, assuming 8-bit samples. Perform
*		the read using individual register read service calls.
*
*	Arguments:
*
*		io			The common I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_read_8_reg(gsc_io_t* io)
{
	U32	bytes;
	U8*	dest;
	U32	status	= (U32) GSC_SUCCESS;
	U32	u32;

	dest	= io->xfer_buf;

	if (io->xfer > _8_BIT_LIMIT_)
		bytes	= _8_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (; bytes--; dest++)
	{
		status	= gsc_bar_reg_read(io->bar, io->reg_adj, &u32);
		if (status != (U32) GSC_SUCCESS)	break;
		dest[0]	= (U8) u32;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_read_16_reg
*
*	Purpose:
*
*		Read the specified number of bytes, assuming 16-bit samples. Perform
*		the read using individual register read service calls.
*
*	Arguments:
*
*		io			The I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_read_16_reg(gsc_io_t* io)
{
	U32		bytes;
	U16*	dest;
	U32		status	= (U32) GSC_SUCCESS;
	U32		u32;

	dest	= io->xfer_buf;

	if (io->xfer > _16_BIT_LIMIT_)
		bytes	= _16_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (bytes /= 2; bytes--; dest++)
	{
		status	= gsc_bar_reg_read(io->bar, io->reg_adj, &u32);
		if (status != (U32) GSC_SUCCESS)	break;
		dest[0]	= (U16) u32;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_read_32_reg
*
*	Purpose:
*
*		Read the specified number of bytes, assuming 32-bit samples. Perform
*		the read using individual register read service calls.
*
*	Arguments:
*
*		io			The I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_read_32_reg(gsc_io_t* io)
{
	U32		bytes;
	U32*	dest;
	U32		status	= (U32) GSC_SUCCESS;
	U32		u32;

	dest	= io->xfer_buf;

	if (io->xfer > _16_BIT_LIMIT_)
		bytes	= _16_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (bytes /= 4; bytes--; dest++)
	{
		status	= gsc_bar_reg_read(io->bar, io->reg_adj, &u32);
		if (status != (U32) GSC_SUCCESS)	break;
		dest[0]	= (U16) u32;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_write_8_reg
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 8-bit samples. Perform
*		the write using individual register write service calls.
*
*	Arguments:
*
*		io			The I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_write_8_reg(gsc_io_t* io)
{
	U32			bytes;
	const U8*	src;
	U32			status	= (U32) GSC_SUCCESS;

	src		= io->xfer_buf;

	if (io->xfer > _8_BIT_LIMIT_)
		bytes	= _8_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (; bytes--; src++)
	{
		status	= gsc_bar_reg_write(io->bar, io->reg_adj, src[0]);
		if (status != (U32) GSC_SUCCESS)	break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_write_16_reg
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 16-bit samples. Perform
*		the write using individual register write service calls.
*
*	Arguments:
*
*		io			The I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_write_16_reg(gsc_io_t* io)
{
	U32			bytes;
	const U16*	src;
	U32			status	= (U32) GSC_SUCCESS;

	src		= io->xfer_buf;

	if (io->xfer > _16_BIT_LIMIT_)
		bytes	= _16_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (bytes /= 2; bytes--; src++)
	{
		status	= gsc_bar_reg_write(io->bar, io->reg_adj, src[0]);
		if (status != (U32) GSC_SUCCESS)	break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	gsc_io_pio_write_32_reg
*
*	Purpose:
*
*		Write the specified number of bytes, assuming 32-bit samples. Perform
*		the write using individual register write service calls.
*
*	Arguments:
*
*		io			The I/O parameters for the request.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_io_pio_write_32_reg(gsc_io_t* io)
{
	U32			bytes;
	const U32*	src;
	U32			status	= (U32) GSC_SUCCESS;

	src		= io->xfer_buf;

	if (io->xfer > _32_BIT_LIMIT_)
		bytes	= _32_BIT_LIMIT_;
	else
		bytes	= io->xfer;

	io->xfered	= bytes;

	for (bytes /= 4; bytes--; src++)
	{
		status	= gsc_bar_reg_write(io->bar, io->reg_adj, src[0]);
		if (status != (U32) GSC_SUCCESS)	break;
	}

	return(status);
}


