// This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL).
// Any Device - Driver - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/driver/close.c $
// $Rev: 23671 $
// $Date: 2013-10-31 16:23:27 -0500 (Thu, 31 Oct 2013) $

#include "main.h"



/******************************************************************************
*
*	Function:	close_dev
*
*	Purpose:
*
*		Close the given device.
*
*	Arguments:
*
*		dev		The device to close.
*
*	Returned:
*
*		0		All went well.
*		< 0		There was a problem.
*
******************************************************************************/

int close_dev(dev_data_t* dev)
{
	gsc_drv_dma_t	dma;
	int				status;
	int				test;

	for (;;)	/* We'll use a loop for convenience. */
	{
		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We didn't get the semaphore. */
			status	= -ERESTARTSYS;
			break;
		}

		/* Close the device. */
		memset(&dma, 0, sizeof(dma));
		dma.index	= 0;
		dma_close_ioctl(dev, &dma);

		memset(&dma, 0, sizeof(dma));
		dma.index	= 1;
		dma_close_ioctl(dev, &dma);

		pid_remove(dev);
		status		= 0;
		MOD_DEC_USE_COUNT;

		if (dev->usage)
			dev->usage--;

		if (dev->usage == 0)
		{
			irq_release(dev);
			writel(0, dev->vaddr.plx_intcsr_32);
		}

		up(&dev->sem);
		status	= 0;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	close
*
*	Purpose:
*
*		Implement the release() procedure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*		filp	This the file structure.
*
*	Returned:
*
*		0		All went well.
*		< 0		There was a problem.
*
******************************************************************************/

int close(struct inode* inode, struct file* filp)
{
	dev_data_t*	dev;
	int			status;

	dev	= dev_locate(inode);

	if (dev == NULL)
	{
		/* The referenced device doesn't exist. */
		status	= -ENODEV;
	}
	else
	{
		/* Perform the work of closing the device. */
		status	= close_dev(dev);
	}

	return(status);
}



