// This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL).
// Any Device - Driver - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/driver/kernel_2_2.c $
// $Rev: 23671 $
// $Date: 2013-10-31 16:23:27 -0500 (Thu, 31 Oct 2013) $
//
//	Provide special handling for things specific to the 2.2 kernel.
//

#include <linux/version.h>



#if	(LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,0)) && \
	(LINUX_VERSION_CODE <  KERNEL_VERSION(2,3,0))

#define	MODULE
#include "main.h"



/* Tags	**********************************************************************/

EXPORT_NO_SYMBOLS;
MODULE_AUTHOR("General Standards Corporation, www.generalstandards.com");
MODULE_DESCRIPTION(	"GSC " DRV_DEV_DEVICE_NAME_UPPER
					" driver " DRV_GSC_FILE_VERSION_STR);
MODULE_SUPPORTED_DEVICE("GSC " DRV_DEV_DEVICE_NAME_UPPER);



/******************************************************************************
*
*	Function:	dma_sgl_user_mem_init
*
*	Purpose:
*
*		Initialize things in preparation for future kernel access of user
*		memory for SGL DMA operations.
*
*	Arguments:
*
*		dma		The DMA structure for which any initialization should be done.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dma_sgl_user_mem_init(dev_dma_t* dma)
{
	// SGL is not yet supported in the 2.2 kernel.
}



/******************************************************************************
*
*	Function:	dma_sgl_user_mem_map
*
*	Purpose:
*
*		Map user memory to kernel space for an SGL DMA transfer.
*
*	Arguments:
*
*		dma		The DMA structure with the references to the user memory
*				whose references are to be freed.
*
*		xfer	The data for the DMA transfer to be performed.
*
*	Returned:
*
*		0		All went well.
*		<0		An error code for the problem seen.
*
******************************************************************************/

int dma_sgl_user_mem_map(dev_dma_t* dma, gsc_drv_dma_t* xfer)
{
	int	status;

	// SGL is not yet supported in the 2.2 kernel.
	status			= -ENOMEM;
	xfer->status	= GSC_UNSUPPORTED_FUNCTION;
	return(status);
}



/******************************************************************************
*
*	Function:	dma_sgl_user_mem_reset
*
*	Purpose:
*
*		Reset things to cleanup for any activity after kernel access of user
*		memory for SGL DMA operations.
*
*	Arguments:
*
*		dma		The DMA structure for which any reset steps should be taken.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dma_sgl_user_mem_reset(dev_dma_t* dma)
{
	// SGL is not yet supported in the 2.2 kernel.
}



/******************************************************************************
*
*	Function:	dma_sgl_user_mem_unmap
*
*	Purpose:
*
*		Free all references to any user memory made for an SGL DMA transfer.
*
*	Arguments:
*
*		dma		The DMA structure with the references to the user memory
*				whose references are to be freed.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void dma_sgl_user_mem_unmap(dev_dma_t* dma)
{
	// SGL is not yet supported in the 2.2 kernel.
}



/******************************************************************************
*
*	Function:	irq_isr
*
*	Purpose:
*
*		Service an interrupt.  For safety sake, each detected interrupt is
*		cleared then disabled.
*
*	Arguments:
*
*		irq		The interrupt number.
*
*		dev_id	The private data we've associated with the IRQ.
*
*		regs	Unused.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void irq_isr(int irq, void* dev_id, struct pt_regs* regs)
{
	irq_isr_common(irq, dev_id);
}



/******************************************************************************
*
*	Function:	mmap_nopage
*
*	Purpose:
*
*		Implement the mmap nopage() procedure.
*
*	Arguments:
*
*		vma				The virtual memory area of interest.
*
*		address			The virtual address of the page being accessed.
*
*		write_access	The "no-share" option.
*	Returned:
*
*		0				The page is invalid.
*		else			The virtual address of the page.
*
******************************************************************************/

unsigned long mmap_nopage(	struct vm_area_struct*	vma,
							unsigned long			address,
							int						write_access)
{
	struct page*	page;
	unsigned long	ul;

	mmap_nopage_common(vma, address, &page);

	if (page == NULL)
		ul	= 0;
	else
		ul	= (u32) __va((page - mem_map) << PAGE_SHIFT);

	return(ul);
}



/******************************************************************************
*
*	Function:	mmap_unmap
*
*	Purpose:
*
*		Implement the mmap unmap() procedure.
*
*	Arguments:
*
*		vma		The virtual memory area of interest.
*
*		addr	The address of the area being released.
*
*		len		The size of the area being released.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void mmap_unmap(struct vm_area_struct*	vma,
				unsigned long			addr,
				size_t				len)
{
	dev_data_t*		dev;
	dev_mem_t*		mem;
	unsigned long	offset;
	dev_pid_t*		pid;
	int			test;

	for (;;)	/* We'll use a loop for convenience. */
	{
		/*
		*	Validate some of the parameters.
		*/

		dev	= dev_locate(vma->vm_file->f_dentry->d_inode);

		if (dev == NULL)
		{
			/* The referenced device doesn't exist. */
			break;
		}

		/*
		*	Gain the neccessary access.
		*/

		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We got a signal rather than the semaphore. */
			break;
		}

		pid	= pid_locate(dev);

		if (pid == NULL)
		{
			/* The process data wasn't found. */
			up(&dev->sem);
			break;
		}

		/*
		*	See which memory area this request applies to.
		*/

		offset	= VM_OFFSET_BYTES(vma);
		mem		= mem_mmap_locate(pid, offset, len, 1);

		if ((mem) && (mem->mmap_users))
			mem->mmap_users--;

		up(&dev->sem);
		break;
	}
}



/******************************************************************************
*
*	Function:	proc_get_info
*
*	Purpose:
*
*		Implement the get_info() service for /proc file system support.
*
*	Arguments:
*
*		page	The data produced is put here.
*
*		start	Records pointer to where the data in "page" begins.
*
*		offset	The offset into the file where we're to begin.
*
*		count	The limit to the amount of data we can produce.
*
*		dummy	A parameter that is unused.
*
*	Returned:
*
*		int		The number of characters written.
*
******************************************************************************/

int proc_get_info(
	char*	page,
	char**	start,
	off_t	offset,
	int		count,
	int		dummy)
{
	int	eof;
	int	i;

	i	= proc_read(page, start, offset, count, &eof, NULL);
	return(i);
}



#endif
