// This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL).
// Any Device - Driver - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/driver/open.c $
// $Rev: 23671 $
// $Date: 2013-10-31 16:23:27 -0500 (Thu, 31 Oct 2013) $

#include "main.h"



/******************************************************************************
*
*	Function:	open
*
*	Purpose:
*
*		Implement the open() procedure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*		filp	This the file structure.
*
*	Returned:
*
*		0		All went well.
*		<0		An error code for the problem seen.
*
******************************************************************************/

int open(struct inode* inode, struct file* filp)
{
	#define	_RW_	(FMODE_READ | FMODE_WRITE)

	dev_data_t*	dev;
	int			status;
	int			test;

	for (;;)	/* We'll use a loop for convenience. */
	{
		if ((filp->f_mode & _RW_) != _RW_)
		{
			/* Read and write access are both required. */
			status	= -EINVAL;
			break;
		}

		dev	= dev_locate(inode);

		if (dev == NULL)
		{
			/* The referenced device doesn't exist. */
			status	= -ENODEV;
			break;
		}

		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We didn't get the semaphore. */
			status	= -ERESTARTSYS;
			break;
		}

		test	= pid_add(dev);

		if (test)
		{
			/* We couldn't add a new process. */
			status	= -EBUSY;
			up(&dev->sem);
			break;
		}

		/* Open the device. */
		if (dev->usage == 0)
		{
			status	= irq_request(dev);

			if (status)
			{
				/* We can't get the interrupt. */
				up(&dev->sem);
				break;
			}
		}

		MOD_INC_USE_COUNT;
		filp->private_data	= dev;
		dev->usage++;
		status		= 0;
		up(&dev->sem);
		break;
	}

	return(status);
}



