// Application - HPDI32 - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/rthrottle/tx.c $
// $Rev: 23669 $
// $Date: 2013-10-31 16:21:56 -0500 (Thu, 31 Oct 2013) $

#include "main.h"



//*****************************************************************************
static int _dev_config(void* dev, int rthrottle, int io_mode)
{
	int				errs		= 0;
	unsigned long	cmd_3;
	unsigned long	start;
	unsigned long	throttle;

	label_text("Tx Configuration");
	printf("\n");
	label_level_inc();

	if (rthrottle)
	{
		cmd_3		= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;
		start		= HPDI32_TX_AUTO_START_NO;
		throttle	= HPDI32_TX_REMOTE_THROTTLE_ENABLE;
	}
	else
	{
		cmd_3		= HPDI32_CABLE_COMMAND_MODE_GPIO_IN;
		start		= HPDI32_TX_AUTO_START_YES;
		throttle	= HPDI32_TX_REMOTE_THROTTLE_DISABLE;
	}

	errs	+= dev_init						(dev, -1, 1);
	errs	+= dev_favor_tx					(dev, -1, 1, HPDI32_MISC_FAVOR_TX_ENABLE, NULL);
	errs	+= dev_tx_rx_tri_state			(dev, -1, 1, HPDI32_MISC_TX_RX_TRI_STATE_DISABLE, NULL);
	errs	+= dev_cable_command_mode_0		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL, NULL);
	errs	+= dev_cable_command_mode_1		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_IN, NULL);
	errs	+= dev_cable_command_mode_2		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_IN, NULL);
	errs	+= dev_cable_command_mode_3		(dev, -1, 1, cmd_3, NULL);
	errs	+= dev_cable_command_mode_4		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_IN, NULL);
	errs	+= dev_cable_command_mode_5		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL, NULL);
	errs	+= dev_cable_command_mode_6		(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_IN, NULL);
	errs	+= dev_tx_auto_start			(dev, -1, 1, start, NULL);
	errs	+= dev_tx_auto_stop				(dev, -1, 1, HPDI32_TX_AUTO_STOP_NO, NULL);
	errs	+= dev_tx_clock_divider			(dev, -1, 1, 0, NULL);
	errs	+= dev_tx_fifo_ae_level			(dev, -1, 1, 64, NULL);
	errs	+= dev_tx_fifo_af_level			(dev, -1, 1, 64, NULL);
	errs	+= dev_tx_io_data_size			(dev, -1, 1, HPDI32_IO_DATA_SIZE_32_BITS, NULL);
	errs	+= dev_tx_io_mode				(dev, -1, 1, io_mode, NULL);
	errs	+= dev_tx_io_dma_control_mode	(dev, -1, 1, HPDI32_IO_DMA_CONTROL_MODE_AUTOMATIC, NULL);
	errs	+= dev_tx_io_dma_channel_sel	(dev, -1, 1, HPDI32_IO_DMA_CHANNEL_SEL_STATIC, NULL);
	errs	+= dev_tx_io_dma_priority		(dev, -1, 1, HPDI32_IO_DMA_PRIORITY_ENABLE, NULL);
	errs	+= dev_tx_io_overlap_enable		(dev, -1, 1, HPDI32_IO_OVERLAP_ENABLE_NO, NULL);
	errs	+= dev_tx_io_pio_threshold		(dev, -1, 1, 32, NULL);
	errs	+= dev_tx_io_single_cycle		(dev, -1, 1, HPDI32_IO_SINGLE_CYCLE_PRESENT, NULL);
	errs	+= dev_tx_io_timeout			(dev, -1, 1, 10, NULL);
	errs	+= dev_tx_line_valid_off_count	(dev, -1, 1, 0, NULL);
	errs	+= dev_tx_line_valid_on_count	(dev, -1, 1, 0, NULL);
	errs	+= dev_tx_remote_throttle		(dev, -1, 1, throttle, NULL);
	errs	+= dev_tx_status_valid_count	(dev, -1, 1, 0, NULL);
	errs	+= dev_tx_status_valid_mirror	(dev, -1, 1, HPDI32_TX_STATUS_VALID_MIRROR_DISABLE, NULL);

	label_level_dec();
	return(errs);
}



//*****************************************************************************
static int _wait_for_rx_enabled(void* dev, int seconds)
{
	long	delta;
	int		errs	= 0;
	time_t	now;
	time_t	start;
	U32		status;
	U32		value;

	label_text("Wait for Rx Enabled");
	start	= time(NULL);

	for (;;)
	{
		now	= time(NULL);

		if (now > (start + seconds))
		{
			errs++;
			printf("FAIL <---  (%d second time out expired)\n", seconds);
			break;
		}

		status	= hpdi32_reg_read(dev, HPDI32_BSR, &value);

		if (status != GSC_SUCCESS)
		{
			errs++;
			printf("FAIL <---  (GPIO read status: 0x%lX)\n", (long) status);
			break;
		}

		if (value & 0x40)	// D6 is the Rx Enabled input: hi = enabled
		{
			delta	= now - start;
			printf("Done  (%ld second%s)\n", delta, (delta == 1) ? "" : "s");
			break;
		}

		usleep(100000);
	}

	return(errs);
}



//*****************************************************************************
static int _wait_for_rx_disabled(void* dev, int seconds)
{
	long	delta;
	int		errs	= 0;
	time_t	now;
	time_t	start;
	U32		status;
	U32		value;

	label_text("Wait for Rx Disabled");
	start	= time(NULL);

	for (;;)
	{
		now	= time(NULL);

		if (now > (start + seconds))
		{
			errs++;
			printf("FAIL <---  (%d second time out expired)\n", seconds);
			break;
		}

		status	= hpdi32_reg_read(dev, HPDI32_BSR, &value);

		if (status != GSC_SUCCESS)
		{
			errs++;
			printf("FAIL <---  (GPIO read status: 0x%lX)\n", (long) status);
			break;
		}

		if (value & 0x40)	// D6 is the Rx Enabled input: hi = enabled
		{
			usleep(100000);
			continue;
		}

		delta	= now - start;
		printf("Done  (%ld second%s)\n", delta, (delta == 1) ? "" : "s");
		break;
	}

	return(errs);
}



//*****************************************************************************
static int _tx_data(void* dev, int block_qty, int api_size, void* api_buf, int errs)
{
	long		blocks	= 0;
	char		buf[64];
	const char*	factor	= "";
	U32			send	= (U32) api_size;
	U32			sent;
	U32			status;
	long long	total	= 0;

	label_text("Transmitting Data");

	if (errs)
	{
		errs	= 0;
		printf("Aborted due to setup errors.\n");
	}
	else
	{
		printf("Working ...");
		label_level_inc();

		for (blocks = 0; block_qty > 0; block_qty--, blocks++)
		{
			if ((blocks % 50) == 0)
			{
				printf("\n");
				sprintf(buf, "Block #%ld", (long) blocks + 1);
				label_text(buf);
			}

			printf(".");
			fflush(stdout);
			status	= hpdi32_write(dev, api_buf, send, &sent);
			total	+= sent;

			if (status != GSC_SUCCESS)
			{
				errs++;
				printf("FAIL <---  (hpdi32_write(), status 0x%lX)\n", (long) status);
				break;
			}

			if (sent != send)
			{
				errs++;
				printf("FAIL <---  (hpdi32_write(), sent too few bytes: %ld)\n", (long) sent);
				break;
			}
		}

		if (errs == 0)
			printf("\n");

		label_text("Results");

		if ((total) && ((total % 1024) == 0))
		{
			total	/= 1024;
			factor	= "K";
		}

		if ((total) && ((total % 1024) == 0))
		{
			total	/= 1024;
			factor	= "M";
		}

		if ((total) && ((total % 1024) == 0))
		{
			total	/= 1024;
			factor	= "G";
		}

		if ((total) && ((total % 1024) == 0))
		{
			total	/= 1024;
			factor	= "T";
		}

		label_long_comma(total);
		printf("%s byte%s (1K = 1024)\n", factor, (total == 1) ? "" : "s");
		label_level_dec();
	}

	return(errs);
}



//*****************************************************************************
static int _data_init(void* dev, unsigned long size, void* buf, int save)
{
	FILE*			file	= NULL;
	long			l;
	const char*		name	= "tx_data.txt";
	char*			ptr		= buf;
	unsigned long	ul;

	label_text("Data Initialization");

	if (buf)
	{
		memset(ptr, 'x', size);
		ptr[size - 2]	= '\r';
		ptr[size - 1]	= '\n';

		for (l = 0, ul = size; ul >= 64; ul -= 64, ptr += 64, l++)
		{
			sprintf(ptr, "%6ld. -- 0x%08lX -- 0x%08lX: ", l % 999999, (long) ul & 0xFFFFFFFF, (long) ptr & 0xFFFFFFFF);
			ptr[62]	= '\r';
			ptr[63]	= '\n';
		}

		printf("Done\n");
	}
	else
	{
		printf("SKIPPED  (no buffer acquired)\n");
	}

	if (save)
	{
		label_text("Saving Data");
		file	= file_create(name);

		if (file)
		{
			file_write(name, file, buf, size);
			file_close(name, file);
			printf("Done\n");
		}
	}

	return(0);
}



//*****************************************************************************
static int _wait_for_tx_empty(void* dev, int seconds)
{
	long			delta;
	int				errs	= 0;
	unsigned long	fifo	= 0;
	time_t			now;
	time_t			start;
	U32				status;

	label_text("Wait for Tx Empty");
	start	= time(NULL);

	for (;;)
	{
		now	= time(NULL);

		if (now > (start + seconds))
		{
			errs++;
			printf("FAIL <---  (%d second time out expired)\n", seconds);
			break;
		}

		status	= HPDI32_FIFO_STATUS__TX_GET(dev, &fifo);

		if (status != GSC_SUCCESS)
		{
			errs++;
			printf("FAIL <---  (Tx FIFO Status read error: 0x%lX)\n", (long) status);
			break;
		}

		if (fifo == HPDI32_FIFO_STATUS_EMPTY)
		{
			delta	= now - start;
			printf("Done  (%ld second%s)\n", delta, (delta == 1) ? "" : "s");
			break;
		}
	}

	return(errs);
}



//*****************************************************************************
int tx_data(void* dev, int rthrottle, int block_size, int block_qty, int save, int io_mode)
{
	void*			api_buf		= NULL;
	unsigned long	api_size	= block_size;
	int				errs		= 0;

	for (;;)	// A convenience loop.
	{
		label_text("Setup");
		printf("\n");
		label_level_inc();
		errs	+= _dev_config(dev, rthrottle, io_mode);
		errs	+= buffer_alloc(dev, &api_size, &api_buf);

		if (block_size <= api_size)
			errs	+= _data_init(dev, block_size, api_buf, save);
		else
			errs	+= _data_init(dev, api_size, api_buf, save);

		// Tx is enabled before Rx is enabled.
		errs	+= dev_tx_enable(dev, -1, 1, HPDI32_TX_ENABLE_YES, NULL);
		errs	+= _wait_for_rx_enabled(dev, 30);
		label_level_dec();

		errs	+= _tx_data(dev, block_qty, block_size, api_buf, errs);

		label_text("Cleanup");
		printf("\n");
		label_level_inc();
		errs	+= _wait_for_tx_empty(dev, 30);

		// Force the Frame Valid line low so no spurious data can be transmitted.
		errs	+= dev_cable_command_mode_0(dev, -1, 1, HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW, NULL);

		// Tx is disabled before Rx is disabled.
		errs	+= dev_tx_enable(dev, -1, 1, HPDI32_TX_ENABLE_NO, NULL);
		errs	+= _wait_for_rx_disabled(dev, 30);

		errs	+= dev_tx_io_buffer_size(dev, -1, 1, 0, NULL);
		label_level_dec();
		break;
	}

	return(errs);
}


