// Application - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/rx/rx.c $
// $Rev: 23668 $
// $Date: 2013-10-31 16:21:36 -0500 (Thu, 31 Oct 2013) $

#include "main.h"

#include <stdio.h>
#include <string.h>



/* variables	*************************************************************/

static	U32	buffer[256L * 1024];	// 1MB of data.



/*****************************************************************************
*
*	Function:	_init
*
*	Purpose:
*
*		Initialize things before we start.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _init(void* handle)
{
	int		errs	= 0;
	U32		status;

	label_text("Initializing");
	memset(buffer, 0, sizeof(buffer));
	status	= hpdi32_init(handle);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "hpdi32_init", status);
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_config
*
*	Purpose:
*
*		Configure things before we start.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*		fifos	Tell the caller if there are FIFOs.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _config(void* handle, int* fifos)
{
	int				errs	= 0;
	unsigned long	feature	= 0;
	unsigned long	size	= 0;
	U32				status;

	label_text("Configuring");

	status	= HPDI32_MISC_TX_RX_TRI_STATE__YES(handle);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "HPDI32_MISC_TX_RX_TRI_STATE__YES", status);

	if (status == GSC_SUCCESS)
	{
		status	= HPDI32_MISC_FAVOR_TX__NO(handle);

		if (status != GSC_SUCCESS)
			errs	+= label_fail_func(1, "HPDI32_MISC_FAVOR_TX__NO", status);
	}

	if (status == GSC_SUCCESS)
	{
		status	= HPDI32_IO_MODE__RX_PIO(handle);

		if (status != GSC_SUCCESS)
			errs	+= label_fail_func(1, "HPDI32_IO_MODE__RX_PIO", status);
	}

	if (status == GSC_SUCCESS)
	{
		status	= HPDI32_FIFO_SIZE__RX_GET(handle, &size);

		if (status != GSC_SUCCESS)
			errs	+= label_fail_func(1, "HPDI32_FIFO_SIZE__RX_GET", status);
	}

	if (status == GSC_SUCCESS)
	{
		status	= HPDI32_MISC_FEATURES__FIFO_SIZE(handle, &feature);

		if (status != GSC_SUCCESS)
		{
			errs	+= label_fail_func(1, "HPDI32_MISC_FEATURES__FIFO_SIZE", status);
		}
		else if ((feature == HPDI32_MISC_FEATURES_PRESENT) && (size == 0))
		{
			fifos[0]	= 0;
			printf("  No FIFOs installed.\n");
		}
	}

	if ((status == GSC_SUCCESS) && (fifos[0]))
	{
		status	= HPDI32_RX_ENABLE__YES(handle);

		if (status != GSC_SUCCESS)
			errs	+= label_fail_func(1, "HPDI32_RX_ENABLE__YES", status);
	}

	if (errs == 0)
		printf("PASS\n");

	return(errs);
}


/*****************************************************************************
*
*	Function:	_rx
*
*	Purpose:
*
*		Configure things before we start.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*		fifos	Are FIFOs installed on the board?
*
*		errs	Have any errors already been reported?
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

static int _rx(void* handle, int fifos, int errs)
{
	char	buf[128];
	size_t	bytes	= sizeof(buffer);
	U32		got		= 0;
	U32		status;

	for (;;)
	{
		sprintf(buf, "Reading %lu Bytes", (long) bytes);
		label_text(buf);

		if (fifos == 0)
		{
			errs	= 0;
			printf("ABORTED <--   (no FIFOS)\n");
			break;
		}

		if (errs)
		{
			errs	= 0;
			printf("ABORTED <--   (setup errors)\n");
			break;
		}

		bytes	= sizeof(buffer);
		status	= hpdi32_read(handle, buffer, sizeof(buffer), &got);

		if ((status == GSC_WAIT_TIMEOUT) && (got == 0))
		{
			errs	= 0;
			printf("ABORTED <--   (No Rx Clock at the cable interface)\n");
			break;
		}

		if (status != GSC_SUCCESS)
		{
			errs	+= label_fail_func(1, "hpdi32_read", status);
			break;
		}

		if (got == 0)
		{
			errs	= 1;
			printf("FAIL <--  (no data returned)\n");
			break;
		}

		printf("PASS  (%lu Bytes)\n", (long) got);
		break;
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	rx_data
*
*	Purpose:
*
*		Perform a simple data receive operation.
*
*	Arguments:
*
*		handle	The handle of the board to access.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
*****************************************************************************/

int	rx_data(void* handle)
{
	int		errs	= 0;
	int		fifos	= 1;

	label_text("Receiving Data");
	printf("\n");
	label_level_inc();

	errs	+= _init(handle);
	errs	+= _config(handle, &fifos);
	errs	+= _rx(handle, fifos, errs);

	label_level_dec();
	return(errs);
}


