// Application - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/sbtest/io.c $
// $Rev: 23667 $
// $Date: 2013-10-31 16:21:12 -0500 (Thu, 31 Oct 2013) $

#include "main.h"

#include <stdio.h>
#include <string.h>
#include <time.h>
#include <asm/types.h>



/* #defines	*****************************************************************/

#define	_1K					(1024L)
#define	_10K				(10L * _1K)
#define	_200K				(200L * _1K)
#define	_1G					(1024L * 1024L * 1024L)

#define	WRITE(h,x)				\
			(_buf_ptr == NULL)	\
			? (GSC_FAILED)		\
			: (hpdi32_write((h), _buf_ptr, _buf_size, &(x)))

#define	READ(h,x)				\
			(_buf_ptr == NULL)	\
			? (GSC_FAILED)		\
			: (hpdi32_read((h), _buf_ptr, _buf_size, &(x)))



/* typedefs	*****************************************************************/

typedef struct
{
	unsigned long	set;
	unsigned long	get;
} set_get_t;

typedef struct
{
	const char*			name;
	U32					parm;
	const set_get_t*	list;
	size_t				size;
} set_get_list_t;

typedef struct
{
	int		tx;				// Is this Tx or Rx?
	U32		fifo_size;		// Size of FIFO in samples.
	U32		buf[1024];		// Samples per transfer.
	U32*	xfer_buf;		// Pointer to current transfer buffer.
	int		pending;		// I/O call is oending.

	long	xfer_b_tot;		// Total bytes to transfer.
	long	xfer_b;			// Bytes to tansfer in this block.

	long	xfered_b_tot;	// Total bytes transferred so far.
	long	xfered_b;		// Total bytes transfered for this block.

} io_t;

typedef struct
{
	long	limit;			// Maximum samples that can be in the pipeline.
	long	current;		// Samples currently in pipeline.
} pipeline_t;



/* variables	*************************************************************/

static	U32				_buffer[150L * 1024];	// Bigger than largest Tx FIFO
static	void*			_buf_ptr				= NULL;
static	unsigned long	_buf_size				= 0;
static	U32				_io_callback_status;



//****************************************************************************
static int _api_buffer_alloc(void* handle)
{
	#define	_2M		(2L * 1024L * 1024L)

	unsigned long	buffer	= 0;
	int				errs	= 0;
	unsigned long	size	= 0;
	int				status;

	status	= HPDI32_IO_BUFFER_SIZE__TX_SET(handle, _2M, &size);

	if (status != GSC_SUCCESS)
	{
		errs	+= label_fail_func(1, "HPDI32_IO_BUFFER_SIZE__TX_SET", status);
	}
	else if (size < _2M)
	{
		errs++;
		printf("FAIL <---  (%d. 2MB buffer size request failed.)\n", __LINE__);
	}

	if (errs == 0)
	{
		status	= HPDI32_IO_BUFFER_POINTER__TX_GET(handle, &buffer);

		if (status != GSC_SUCCESS)
		{
			errs	+= label_fail_func(1, "HPDI32_IO_BUFFER_POINTER__TX_GET", status);
		}
		else if (buffer == 0)
		{
			errs++;
			printf("FAIL <---  (%d. Failed to acquire buffer pointer.)\n", __LINE__);
		}
		else
		{
			memset((void*) buffer, 0, size);
		}
	}

	_buf_ptr	= (void*) buffer;
	_buf_size	= size;
	return(errs);
}



//****************************************************************************
static int _api_buffer_free(void* handle)
{
	int	errs	= 0;
	U32	status;

	status	= HPDI32_IO_BUFFER_SIZE__TX_FREE(handle);

	if (status != GSC_SUCCESS)
		errs	+= label_fail_func(1, "HPDI32_IO_BUFFER_SIZE__TX_FREE", status);

	_buf_ptr	= (void*) NULL;
	_buf_size	= 0;
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_parm_list_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	Which I/O size are these tests for?
*
*		set		The data set to act on.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_parm_list_test(void* handle, U32 which, const set_get_list_t* set)
{
	int				errs	= 0;
	int				form;
	unsigned long	get;
	int				i;
	char			name[128];
	U32				status	= GSC_SUCCESS;

	sprintf(name, "%s %s", (which & HPDI32_WHICH_TX) ? "Tx" : "Rx", set->name);
	label_text(name);

	for (form = 0; form <= 2; form++)
	{
		printf(".");

		for (i = 0; i < (int) set->size; i++)
		{
			status	= hpdi32_config(handle,
									set->parm,
									which,
									set->list[i].set,
									&get);

			if ((status == GSC_SUCCESS) && (get != set->list[i].get))
				status	= GSC_FAILED;

			if (status != GSC_SUCCESS)
				break;
		}

		if (status != GSC_SUCCESS)
			break;
	}

	hpdi32_reset(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_abort_tests
*
*	Purpose:
*
*		Perform I/O Abort tests. This test doesn't actually verify that the
*		abort request actually works. This test simply verifies that the API
*		service operates properly when an I/O operation is not active.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_abort_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set						get						*/
		{ HPDI32_IO_ABORT_NO,		HPDI32_IO_ABORT_NO		},
		{ GSC_NO_CHANGE,			HPDI32_IO_ABORT_NO		},
		{ GSC_DEFAULT,				HPDI32_IO_ABORT_DEFAULT	},
		{ HPDI32_IO_ABORT_YES,		HPDI32_IO_ABORT_NO		},	// It auto clears.
		{ GSC_NO_CHANGE,			HPDI32_IO_ABORT_NO		},	// It auto clears.
		{ GSC_DEFAULT,				HPDI32_IO_ABORT_DEFAULT	},
		{ HPDI32_IO_ABORT_DEFAULT,	HPDI32_IO_ABORT_DEFAULT	},
		{ GSC_NO_CHANGE,			HPDI32_IO_ABORT_DEFAULT	},
		{ GSC_DEFAULT,				HPDI32_IO_ABORT_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Abort",
		/* parm	*/	HPDI32_IO_ABORT,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_buffer_pointer_test
*
*	Purpose:
*
*		Perform a I/O Buffer Pointer test.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	transmitter or receiver side?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_buffer_pointer_test(void* handle, U32 which)
{
	int				errs	= 0;
	int				form;
	unsigned long	get;
	int				i;
	char			label[128];
	U8*				ptr;
	U32				status;

	sprintf(label,
			"%s Buffer pointer",
			(which & HPDI32_WHICH_TX) ? "Tx" : "Rx");
	label_text(label);

	for (form = 0; form <= 2; form++)
	{
		printf(".");
		status	= hpdi32_init(handle);

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_IO_BUFFER_SIZE,
									which,
									0,
									&get);

			if ((status == GSC_SUCCESS) && (get != 0))
				status	= GSC_FAILED;	// The size should match.
		}

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_IO_BUFFER_POINTER,
									which,
									GSC_NO_CHANGE,
									&get);

			if ((status == GSC_SUCCESS) && (get))
				status	= GSC_FAILED;	// The pointer should be NULL.
		}

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_IO_BUFFER_SIZE,
									which,
									1024,
									&get);

			// The size may be larger than requested, depending on the OS.

			if ((status == GSC_SUCCESS) && (get < 1024))
				status	= GSC_FAILED;	// The size should match.
		}

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_IO_BUFFER_POINTER,
									which,
									GSC_NO_CHANGE,
									&get);

			if ((status == GSC_SUCCESS) && (get == (unsigned long) NULL))
				status	= GSC_FAILED;	// The pointer should NOT be NULL.
		}

		if (status == GSC_SUCCESS)
		{
			ptr	= (U8*) get;

			for (i = 0; i < (int) 1024; i++)
			{
				ptr[0]	= (U8) i;

				if (ptr[0] != (U8) i)
				{
					status	= GSC_FAILED;
					break;
				}
			}
		}
	}


	hpdi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_buffer_pointer_tests
*
*	Purpose:
*
*		Perform I/O Buffer Pointer tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_buffer_pointer_tests(void* handle)
{
	int	errs	= 0;

	errs	+= _io_buffer_pointer_test(handle, HPDI32_WHICH_TX);
	errs	+= _io_buffer_pointer_test(handle, HPDI32_WHICH_RX);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_data_size_tests
*
*	Purpose:
*
*		Perform I/O Data Size tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_data_size_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set							get							*/
		{ HPDI32_IO_DATA_SIZE_8_BITS,	HPDI32_IO_DATA_SIZE_8_BITS	},
		{ GSC_NO_CHANGE,				HPDI32_IO_DATA_SIZE_8_BITS	},
		{ GSC_DEFAULT,					HPDI32_IO_DATA_SIZE_DEFAULT	},
		{ HPDI32_IO_DATA_SIZE_16_BITS,	HPDI32_IO_DATA_SIZE_16_BITS	},
		{ GSC_NO_CHANGE,				HPDI32_IO_DATA_SIZE_16_BITS	},
		{ GSC_DEFAULT,					HPDI32_IO_DATA_SIZE_DEFAULT	},
		{ HPDI32_IO_DATA_SIZE_32_BITS,	HPDI32_IO_DATA_SIZE_32_BITS	},
		{ GSC_NO_CHANGE,				HPDI32_IO_DATA_SIZE_32_BITS	},
		{ GSC_DEFAULT,					HPDI32_IO_DATA_SIZE_DEFAULT	},
		{ HPDI32_IO_DATA_SIZE_DEFAULT,	HPDI32_IO_DATA_SIZE_DEFAULT	},
		{ GSC_NO_CHANGE,				HPDI32_IO_DATA_SIZE_DEFAULT	},
		{ GSC_DEFAULT,					HPDI32_IO_DATA_SIZE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Data Size",
		/* parm	*/	HPDI32_IO_DATA_SIZE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_dma_channel_select_tests
*
*	Purpose:
*
*		Perform I/O DMA Channel Select tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_dma_channel_select_tests(void* handle)
{
	static const set_get_t	tx_list[]	=
	{
		/* set									get										*/
		{ HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC,	HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT	},
		{ HPDI32_IO_DMA_CHANNEL_SEL_STATIC,		HPDI32_IO_DMA_CHANNEL_SEL_STATIC		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CHANNEL_SEL_STATIC		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT	},
		{ HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT,	HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT	},
		{ HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT,	HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT	},
	};

	static const set_get_t	rx_list[]	=
	{
		/* set									get										*/
		{ HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC,	HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CHANNEL_SEL_DYNAMIC		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT	},
		{ HPDI32_IO_DMA_CHANNEL_SEL_STATIC,		HPDI32_IO_DMA_CHANNEL_SEL_STATIC		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CHANNEL_SEL_STATIC		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT	},
		{ HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT,	HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT	},
		{ HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT,	HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CHANNEL_SEL_TX_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CHANNEL_SEL_RX_DEFAULT	},
	};

	static const set_get_list_t	tx_set	=
	{
		/* name	*/	"DMA Chennel Select",
		/* parm	*/	HPDI32_IO_DMA_CHANNEL_SEL,
		/* list	*/	tx_list,
		/* size	*/	ARRAY_ELEMENTS(tx_list)
	};

	static const set_get_list_t	rx_set	=
	{
		/* name	*/	"DMA Chennel Select",
		/* parm	*/	HPDI32_IO_DMA_CHANNEL_SEL,
		/* list	*/	rx_list,
		/* size	*/	ARRAY_ELEMENTS(rx_list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &tx_set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &rx_set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_dma_control_mode_tests
*
*	Purpose:
*
*		Perform I/O DMA Control Mode tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_dma_control_mode_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set									get										*/
		{ HPDI32_IO_DMA_CONTROL_MODE_MANUAL,	HPDI32_IO_DMA_CONTROL_MODE_MANUAL		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CONTROL_MODE_MANUAL		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CONTROL_MODE_DEFAULT		},
		{ HPDI32_IO_DMA_CONTROL_MODE_AUTOMATIC,	HPDI32_IO_DMA_CONTROL_MODE_AUTOMATIC	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CONTROL_MODE_AUTOMATIC	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CONTROL_MODE_DEFAULT		},
		{ HPDI32_IO_DMA_CONTROL_MODE_DEFAULT,	HPDI32_IO_DMA_CONTROL_MODE_DEFAULT		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_CONTROL_MODE_DEFAULT		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_CONTROL_MODE_DEFAULT		}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"DMA Control Mode",
		/* parm	*/	HPDI32_IO_DMA_CONTROL_MODE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_dma_priority_tests
*
*	Purpose:
*
*		Perform I/O DMA Priority tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_dma_priority_tests(void* handle)
{
	static const set_get_t	tx_list[]	=
	{
		/* set									get									*/
		{ HPDI32_IO_DMA_PRIORITY_DISABLE,		HPDI32_IO_DMA_PRIORITY_DISABLE		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_PRIORITY_DISABLE		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_PRIORITY_TX_DEFAULT	},
		{ HPDI32_IO_DMA_PRIORITY_ENABLE,		HPDI32_IO_DMA_PRIORITY_ENABLE		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_PRIORITY_ENABLE		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_PRIORITY_TX_DEFAULT	},
		{ HPDI32_IO_DMA_PRIORITY_RX_DEFAULT,	HPDI32_IO_DMA_PRIORITY_RX_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_PRIORITY_RX_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_PRIORITY_TX_DEFAULT	},
		{ HPDI32_IO_DMA_PRIORITY_TX_DEFAULT,	HPDI32_IO_DMA_PRIORITY_TX_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_PRIORITY_TX_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_PRIORITY_TX_DEFAULT	}
	};

	static const set_get_t	rx_list[]	=
	{
		/* set									get									*/
		{ HPDI32_IO_DMA_PRIORITY_DISABLE,		HPDI32_IO_DMA_PRIORITY_DISABLE		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_PRIORITY_DISABLE		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_PRIORITY_RX_DEFAULT	},
		{ HPDI32_IO_DMA_PRIORITY_ENABLE,		HPDI32_IO_DMA_PRIORITY_ENABLE		},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_PRIORITY_ENABLE		},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_PRIORITY_RX_DEFAULT	},
		{ HPDI32_IO_DMA_PRIORITY_RX_DEFAULT,	HPDI32_IO_DMA_PRIORITY_RX_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_PRIORITY_RX_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_PRIORITY_RX_DEFAULT	},
		{ HPDI32_IO_DMA_PRIORITY_TX_DEFAULT,	HPDI32_IO_DMA_PRIORITY_TX_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IO_DMA_PRIORITY_TX_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IO_DMA_PRIORITY_RX_DEFAULT	}
	};

	static const set_get_list_t	tx_set	=
	{
		/* name	*/	"DMA Priority",
		/* parm	*/	HPDI32_IO_DMA_PRIORITY,
		/* list	*/	tx_list,
		/* size	*/	ARRAY_ELEMENTS(tx_list)
	};

	static const set_get_list_t	rx_set	=
	{
		/* name	*/	"DMA Priority",
		/* parm	*/	HPDI32_IO_DMA_PRIORITY,
		/* list	*/	rx_list,
		/* size	*/	ARRAY_ELEMENTS(rx_list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &tx_set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &rx_set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_buffer_size_list_test
*
*	Purpose:
*
*		Perform a series of tests on the buffer size parameter per the given
*		data and list. While we can request one size, we may be given a size
*		that is different (larger) than we asked for, depending on the OS.
*		We'll take any other condition as a failure since so little is being
*		requested.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	Which I/O size are these tests for?
*
*		set		The data set to act on.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_buffer_size_list_test(void* handle, U32 which, const set_get_list_t* set)
{
	int				errs	= 0;
	int				form;
	unsigned long	get;
	int				i;
	char			name[128];
	U32				status	= GSC_SUCCESS;

	sprintf(name, "%s %s", (which & HPDI32_WHICH_TX) ? "Tx" : "Rx", set->name);
	label_text(name);

	for (form = 0; form <= 2; form++)
	{
		printf(".");

		for (i = 0; i < (int) set->size; i++)
		{
			status	= hpdi32_config(handle,
									set->parm,
									which,
									set->list[i].set,
									&get);

			if (status == GSC_SUCCESS)
			{
				if (set->list[i].get == 0)
				{
					// The size we get should also be zero.

					if (get)
						status	= GSC_FAILED;
				}
				else if (get < set->list[i].get)
				{
					status	= GSC_FAILED;
				}
			}

			if (status != GSC_SUCCESS)
				break;
		}

		if (status != GSC_SUCCESS)
			break;
	}

	hpdi32_reset(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_buffer_size_tests
*
*	Purpose:
*
*		Perform I/O Buffer Size tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_buffer_size_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ HPDI32_IO_BUFFER_SIZE_DEFAULT,	HPDI32_IO_BUFFER_SIZE_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_IO_BUFFER_SIZE_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_IO_BUFFER_SIZE_DEFAULT	},
		{ 1024,								1024							},
		{ GSC_NO_CHANGE,					1024							},
		{ GSC_DEFAULT,						HPDI32_IO_BUFFER_SIZE_DEFAULT	},
		{ 2048,								2048							},
		{ GSC_NO_CHANGE,					2048							},
		{ GSC_DEFAULT,						HPDI32_IO_BUFFER_SIZE_DEFAULT	},
		{ 0,								0								},
		{ GSC_NO_CHANGE,					HPDI32_IO_BUFFER_SIZE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Buffer Size",
		/* parm	*/	HPDI32_IO_BUFFER_SIZE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_buffer_size_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_buffer_size_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_callback_arg_tests
*
*	Purpose:
*
*		Perform I/O Callback Argument tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_callback_arg_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ 1024,								1024							},
		{ GSC_NO_CHANGE,					1024							},
		{ GSC_DEFAULT,						HPDI32_IO_CALLBACK_ARG_DEFAULT	},
		{ 0,								0								},
		{ GSC_NO_CHANGE,					0								},
		{ GSC_DEFAULT,						HPDI32_IO_CALLBACK_ARG_DEFAULT	},
		{ 2048,								2048							},
		{ GSC_NO_CHANGE,					2048							},
		{ GSC_DEFAULT,						HPDI32_IO_CALLBACK_ARG_DEFAULT	},
		{ HPDI32_IO_CALLBACK_ARG_DEFAULT,	HPDI32_IO_CALLBACK_ARG_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_IO_CALLBACK_ARG_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_IO_CALLBACK_ARG_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Callback Argument",
		/* parm	*/	HPDI32_IO_CALLBACK_ARG,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_callback_func_tests
*
*	Purpose:
*
*		Perform I/O Callback Function tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static void _callback(U32 arg1, U32 arg2, U32 arg3) {}	/*lint !e715 */

static int _io_callback_func_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ (unsigned long) _callback,		(unsigned long) _callback		},
		{ GSC_NO_CHANGE,					(unsigned long) _callback		},
		{ GSC_DEFAULT,						HPDI32_IO_CALLBACK_FUNC_DEFAULT	},
		{ 0,								0								},
		{ GSC_NO_CHANGE,					0								},
		{ GSC_DEFAULT,						HPDI32_IO_CALLBACK_FUNC_DEFAULT	},
		{ HPDI32_IO_CALLBACK_FUNC_DEFAULT,	HPDI32_IO_CALLBACK_FUNC_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_IO_CALLBACK_FUNC_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_IO_CALLBACK_FUNC_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Callback Function",
		/* parm	*/	HPDI32_IO_CALLBACK_FUNC,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_mode_tests
*
*	Purpose:
*
*		Perform I/O Mode tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_mode_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set						get						*/
		{ HPDI32_IO_MODE_PIO,		HPDI32_IO_MODE_PIO		},
		{ GSC_NO_CHANGE,			HPDI32_IO_MODE_PIO		},
		{ GSC_DEFAULT,				HPDI32_IO_MODE_DEFAULT	},
		{ HPDI32_IO_MODE_DMA,		HPDI32_IO_MODE_DMA		},
		{ GSC_NO_CHANGE,			HPDI32_IO_MODE_DMA		},
		{ GSC_DEFAULT,				HPDI32_IO_MODE_DEFAULT	},
		{ HPDI32_IO_MODE_DMDMA,		HPDI32_IO_MODE_DMDMA	},
		{ GSC_NO_CHANGE,			HPDI32_IO_MODE_DMDMA	},
		{ GSC_DEFAULT,				HPDI32_IO_MODE_DEFAULT	},
		{ HPDI32_IO_MODE_DEFAULT,	HPDI32_IO_MODE_DEFAULT	},
		{ GSC_NO_CHANGE,			HPDI32_IO_MODE_DEFAULT	},
		{ GSC_DEFAULT,				HPDI32_IO_MODE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Mode",
		/* parm	*/	HPDI32_IO_MODE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_overlap_enable_tests
*
*	Purpose:
*
*		Perform I/O Overlap Enable tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_overlap_enable_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set								get									*/
		{ HPDI32_IO_OVERLAP_ENABLE_NO,		HPDI32_IO_OVERLAP_ENABLE_NO			},
		{ GSC_NO_CHANGE,					HPDI32_IO_OVERLAP_ENABLE_NO			},
		{ GSC_DEFAULT,						HPDI32_IO_OVERLAP_ENABLE_DEFAULT	},
		{ HPDI32_IO_OVERLAP_ENABLE_YES,		HPDI32_IO_OVERLAP_ENABLE_YES		},
		{ GSC_NO_CHANGE,					HPDI32_IO_OVERLAP_ENABLE_YES		},
		{ GSC_DEFAULT,						HPDI32_IO_OVERLAP_ENABLE_DEFAULT	},
		{ HPDI32_IO_OVERLAP_ENABLE_DEFAULT,	HPDI32_IO_OVERLAP_ENABLE_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_IO_OVERLAP_ENABLE_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_IO_OVERLAP_ENABLE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Overlap Enable",
		/* parm	*/	HPDI32_IO_OVERLAP_ENABLE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_single_cycle_tests
*
*	Purpose:
*
*		Perform I/O Single Cycle tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_single_cycle_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ HPDI32_IO_SINGLE_CYCLE_ABSENT,	HPDI32_IO_SINGLE_CYCLE_ABSENT	},
		{ GSC_NO_CHANGE,					HPDI32_IO_SINGLE_CYCLE_ABSENT	},
		{ GSC_DEFAULT,						HPDI32_IO_SINGLE_CYCLE_DEFAULT	},
		{ HPDI32_IO_SINGLE_CYCLE_PRESENT,	HPDI32_IO_SINGLE_CYCLE_PRESENT	},
		{ GSC_NO_CHANGE,					HPDI32_IO_SINGLE_CYCLE_PRESENT	},
		{ GSC_DEFAULT,						HPDI32_IO_SINGLE_CYCLE_DEFAULT	},
		{ HPDI32_IO_SINGLE_CYCLE_DEFAULT,	HPDI32_IO_SINGLE_CYCLE_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_IO_SINGLE_CYCLE_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_IO_SINGLE_CYCLE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Single Cycle",
		/* parm	*/	HPDI32_IO_SINGLE_CYCLE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_status_which_tests
*
*	Purpose:
*
*		Perform I/O Status tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_status_which_tests(void* handle, U32 which)
{
	int				errs	= 0;
	int				form;
	unsigned long	get;
	char			label[128];
	U32				status;
	U32				timeout_s	= 1;
	U32				xfer;

	sprintf(label, "%s I/O Status", (which & HPDI32_WHICH_TX) ? "Tx" : "Rx");
	label_text(label);

	for (form = 0; form <= 1; form++)	// Don't call hpdi32_io_config() with active I/O.
	{
		printf(".");
		status	= hpdi32_init(handle);

		if ((which & HPDI32_WHICH_TX) && (status == GSC_SUCCESS))
			status	= HPDI32_TX_CLOCK_DIVIDER__SET(handle, 1000);

		if (status == GSC_SUCCESS)
		{
			status	= HPDI32_IO_OVERLAP_ENABLE__SET(
								handle,
								which,
								HPDI32_IO_OVERLAP_ENABLE_YES);
		}

		if (status == GSC_SUCCESS)
			status	= HPDI32_IO_MODE__SET(handle, which, HPDI32_IO_MODE_PIO);

		if (status == GSC_SUCCESS)
			status	= HPDI32_IO_TIMEOUT__SET(handle, which, timeout_s);

		errs	+= _api_buffer_alloc(handle);

		if (status != GSC_SUCCESS)
			;
		else if (which & HPDI32_WHICH_TX)
			status	= WRITE(handle, xfer);
		else
			status	= READ(handle, xfer);

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_IO_STATUS,
									which,
									GSC_NO_CHANGE,
									&get);
		}

		// The status should either be ACTIVE or TIMEOUT.

		if ((status == GSC_SUCCESS) &&
			((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ACTIVE))
		{
			// The I/O is active, which is OK. Lets wait for completion.
			// Wait for the timeout to expire.
			sleep_ms(timeout_s + 2000);
		}

		if (status == GSC_SUCCESS)
		{
			status	= hpdi32_config(handle,
									HPDI32_IO_STATUS,
									which,
									GSC_NO_CHANGE,
									&get);
		}

		if (status == GSC_SUCCESS)
		{
			if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ACTIVE)
			{
				printf("(%d. still active) ", __LINE__);
				status	= GSC_FAILED;
			}
			else if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_SUCCESS)
			{
				printf("(%d. unexpected success) ", __LINE__);
				status	= GSC_FAILED;
			}
			else if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ERROR)
			{
				printf("(%d. unexpected error) ", __LINE__);
				status	= GSC_FAILED;
			}
			else if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ABORTED)
			{
				printf("(%d. unexpected abort) ", __LINE__);
				status	= GSC_FAILED;
			}
			else if ((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_TIMEOUT)
			{
				// This is what it should be.
			}
			else
			{
				printf("(%d. didn't timeout) ", __LINE__);
				status	= GSC_FAILED;
			}
		}

		hpdi32_init(handle);
		errs	= (status == GSC_SUCCESS) ? 0 : 1;

		errs	+= _api_buffer_free(handle);

		if (errs)
			break;
	}

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_status_tests
*
*	Purpose:
*
*		Perform I/O Status tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_status_tests(void* handle)
{
	int	errs	= 0;

	errs	+= _io_status_which_tests(handle, HPDI32_WHICH_TX);
	errs	+= _io_status_which_tests(handle, HPDI32_WHICH_RX);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_pio_threshold_tests
*
*	Purpose:
*
*		Perform I/O PIO Threshold tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_pio_threshold_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ HPDI32_IO_PIO_THRESHOLD_NONE,		HPDI32_IO_PIO_THRESHOLD_NONE	},
		{ GSC_NO_CHANGE,					HPDI32_IO_PIO_THRESHOLD_NONE	},
		{ GSC_DEFAULT,						HPDI32_IO_PIO_THRESHOLD_DEFAULT	},
		{ 1024,								1024							},
		{ GSC_NO_CHANGE,					1024							},
		{ GSC_DEFAULT,						HPDI32_IO_PIO_THRESHOLD_DEFAULT	},
		{ HPDI32_IO_PIO_THRESHOLD_DEFAULT,	HPDI32_IO_PIO_THRESHOLD_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_IO_PIO_THRESHOLD_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_IO_PIO_THRESHOLD_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"PIO Threshold",
		/* parm	*/	HPDI32_IO_PIO_THRESHOLD,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_timeout_tests
*
*	Purpose:
*
*		Perform I/O Timeout tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_timeout_tests(void* handle)
{
	static const set_get_t	list[]	=
	{
		/* set							get							*/
		{ HPDI32_IO_TIMEOUT_NO_WAIT,	HPDI32_IO_TIMEOUT_NO_WAIT	},
		{ GSC_NO_CHANGE,				HPDI32_IO_TIMEOUT_NO_WAIT	},
		{ GSC_DEFAULT,					HPDI32_IO_TIMEOUT_DEFAULT	},
		{ HPDI32_IO_TIMEOUT_MAX,		HPDI32_IO_TIMEOUT_MAX		},
		{ GSC_NO_CHANGE,				HPDI32_IO_TIMEOUT_MAX		},
		{ GSC_DEFAULT,					HPDI32_IO_TIMEOUT_DEFAULT	},
		{ HPDI32_IO_TIMEOUT_DEFAULT,	HPDI32_IO_TIMEOUT_DEFAULT	},
		{ GSC_NO_CHANGE,				HPDI32_IO_TIMEOUT_DEFAULT	},
		{ GSC_DEFAULT,					HPDI32_IO_TIMEOUT_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Timeout",
		/* parm	*/	HPDI32_IO_TIMEOUT,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs	= 0;

	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_TX, &set);
	errs	+= _io_parm_list_test(handle, HPDI32_WHICH_RX, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	io_config_tests
*
*	Purpose:
*
*		Perform I/O configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/
extern long sub_test;

int io_config_tests(void* handle)
{
	int	errs	= 0;

	label_text("I/O Configuration");
	printf("\n");
	label_level_inc();

	errs	+= _io_abort_tests(handle);
	errs	+= _io_buffer_pointer_tests(handle);
	errs	+= _io_buffer_size_tests(handle);
	errs	+= _io_callback_arg_tests(handle);
	errs	+= _io_callback_func_tests(handle);
	errs	+= _io_data_size_tests(handle);
	errs	+= _io_dma_channel_select_tests(handle);
	errs	+= _io_dma_control_mode_tests(handle);
	errs	+= _io_dma_priority_tests(handle);
	errs	+= _io_mode_tests(handle);
	errs	+= _io_overlap_enable_tests(handle);
	errs	+= _io_pio_threshold_tests(handle);
	errs	+= _io_single_cycle_tests(handle);
	errs	+= _io_status_tests(handle);
	errs	+= _io_timeout_tests(handle);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_callback
*
*	Purpose:
*
*		Respond to an I/O completion callback.
*
*	Arguments:
*
*		arg1	This is a device handle.
*
*		arg2	The status of the transfer.
*
*		arg3	A user supplied argument. We ignore this.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static void _io_callback(U32 arg1, U32 arg2, U32 arg3)
{
	_io_callback_status	= arg2;
}	/*lint !e715 */



/*****************************************************************************
*
*	Function:	_io_callback_test
*
*	Purpose:
*
*		Perform I/O Completion Notification tests using the Callback feature.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_callback_test(void* handle)
{
	int				errs	= 0;
	unsigned long	get		= 0;
	int				i;
	U32				status;
	U32				xfer;

	memset(_buffer, 0, sizeof(_buffer));
	_io_callback_status	= 0;
	label_text("Callback");
	status	= hpdi32_init(handle);

	errs	+= _api_buffer_alloc(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_MODE__TX_PIO(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_OVERLAP_ENABLE__TX_YES(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_CALLBACK_FUNC__TX_SET(handle, (unsigned long) _io_callback);

	if (status == GSC_SUCCESS)
		status	= HPDI32_TX_ENABLE__YES(handle);

	/*
	*	Things are configured such that ...
	*	1. The actual data transfer out the cable will take a long time.
	*	2. The write() request will timeout after one second.
	*	The result is that the wait request should complete successfully.
	*/

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_TIMEOUT__TX_SET(handle, 1);

	if (status == GSC_SUCCESS)
		status	= HPDI32_TX_CLOCK_DIVIDER__SET(handle, 0xFFFF);

	if (status == GSC_SUCCESS)
		status	= WRITE(handle, xfer);

	if (status == GSC_SUCCESS)
		sleep_ms(2000);				// Allow sufficient callback time.

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_STATUS__TX_GET(handle, &get);

	if ((status == GSC_SUCCESS) && (_io_callback_status == 0))
	{
		printf("(%d. no callback) ", __LINE__);
		status	= GSC_FAILED;
	}

	// 1. Test status from HPDI32_IO_STATUS__TX_GET().
	// 2. Test status passed to the callback.

	for (i = 1; (status == GSC_SUCCESS) && (i <= 2); i++)
	{
		if ((status == GSC_SUCCESS) &&
			((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ACTIVE))
		{
			printf("(%d. i %d, still active) ", __LINE__, i);
			status	= GSC_FAILED;
		}

		if ((status == GSC_SUCCESS) &&
			((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_SUCCESS))
		{
			printf("(%d. i %d, unexpected success) ", __LINE__, i);
			status	= GSC_FAILED;
		}

		if ((status == GSC_SUCCESS) &&
			((get & GSC_IO_STATUS_MASK) != GSC_IO_STATUS_TIMEOUT))
		{
			printf("(%d. i %d, did not timeout) ", __LINE__, i);
			status	= GSC_FAILED;
		}

		if ((status == GSC_SUCCESS) &&
			((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ERROR))
		{
			printf("(%d. i %d, transfer error) ", __LINE__, i);
			status	= GSC_FAILED;
		}

		if ((status == GSC_SUCCESS) &&
			((get & GSC_IO_STATUS_MASK) == GSC_IO_STATUS_ABORTED))
		{
			printf("(%d. i %d, transfer aborted) ", __LINE__, i);
			status	= GSC_FAILED;
		}

		get	= _io_callback_status;
	}

	hpdi32_init(handle);
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	errs	+= _api_buffer_free(handle);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_io_wait_test
*
*	Purpose:
*
*		Perform I/O Completion Notification tests using the Wait feature.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _io_wait_test(void* handle)
{
	int	errs	= 0;
	U32	status;
	U32	xfer;

	memset(_buffer, 0, sizeof(_buffer));
	label_text("Wait");
	status	= hpdi32_init(handle);

	errs	+= _api_buffer_alloc(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_MODE__TX_PIO(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_OVERLAP_ENABLE__TX_YES(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_TX_ENABLE__YES(handle);

	/*
	*	Things are configured such that ...
	*	1. The actual data transfer out the cable will take a long time.
	*	2. The write() request will timeout after one second.
	*	The result is that the wait request should complete successfully.
	*/

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_TIMEOUT__TX_SET(handle, 1);

	if (status == GSC_SUCCESS)
		status	= HPDI32_TX_CLOCK_DIVIDER__SET(handle, 1000);

	if (status == GSC_SUCCESS)
		status	= WRITE(handle, xfer);

	if (status == GSC_SUCCESS)
	{
		status	= hpdi32_io_wait(handle, HPDI32_WHICH_TX, 1000);

		if (status == GSC_WAIT_TIMEOUT)
			status	= GSC_SUCCESS;
	}

	hpdi32_init(handle);
	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	errs	+= _api_buffer_free(handle);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	io_notify_tests
*
*	Purpose:
*
*		Perform I/O notification tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int io_notify_tests(void* handle)
{
	int	errs	= 0;

	label_text("I/O Complete Notification");
	printf("\n");
	label_level_inc();

	errs	+= _io_callback_test(handle);
	errs	+= _io_wait_test(handle);

	label_level_dec();
	return(errs);
}


