// Application - HPDI32 - Any OS
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/sbtest/irq.c $
// $Rev: 23667 $
// $Date: 2013-10-31 16:21:12 -0500 (Thu, 31 Oct 2013) $

#include "main.h"

#include <stdio.h>
#include <string.h>



/* typedefs	*****************************************************************/

typedef struct
{
	unsigned long	set;
	unsigned long	get;
} set_get_t;

typedef struct
{
	const char*			name;
	U32					parm;
	const set_get_t*	list;
	size_t				size;
} set_get_list_t;



/* variables	*************************************************************/

static	U32	_irq_callback_arg2;



/*****************************************************************************
*
*	Function:	_get_irq_name
*
*	Purpose:
*
*		Return the name of the given IRQ.
*
*	Arguments:
*
*		which			The IRQ if interest.
*
*	Returned:
*
*		const char*	The name of the IRQ.
*
*****************************************************************************/

static const char* _get_irq_name(U32 which)
{
	const char*	name	= "NONE";

	if (which & HPDI32_WHICH_IRQ_C0A_)		name	= "Command 0 Active";
	if (which & HPDI32_WHICH_IRQ_C0I_)		name	= "Command 0 Inactive";
	if (which & HPDI32_WHICH_IRQ_C1_)		name	= "Command 1";
	if (which & HPDI32_WHICH_IRQ_C2_)		name	= "Command 2";
	if (which & HPDI32_WHICH_IRQ_C3_)		name	= "Command 3";
	if (which & HPDI32_WHICH_IRQ_C4_)		name	= "Command 4";
	if (which & HPDI32_WHICH_IRQ_C5_)		name	= "Command 5";
	if (which & HPDI32_WHICH_IRQ_C6_)		name	= "Command 6";
	if (which & HPDI32_WHICH_IRQ_TX_E)		name	= "Tx FIFO Empty";
	if (which & HPDI32_WHICH_IRQ_TX_AE)		name	= "Tx FIFO Almost Empty";
	if (which & HPDI32_WHICH_IRQ_TX_AF)		name	= "Tx FIFO Almost Full";
	if (which & HPDI32_WHICH_IRQ_TX_F)		name	= "Tx FIFO Full";
	if (which & HPDI32_WHICH_IRQ_RX_E)		name	= "Rx FIFO Empty";
	if (which & HPDI32_WHICH_IRQ_RX_AE)		name	= "Rx FIFO Almost Empty";
	if (which & HPDI32_WHICH_IRQ_RX_AF)		name	= "Rx FIFO Almost Full";
	if (which & HPDI32_WHICH_IRQ_RX_F)		name	= "Rx FIFO Full";

	return(name);
}



/*****************************************************************************
*
*	Function:	_irq_parm_list_test
*
*	Purpose:
*
*		Perform a series of tests per the given data and list.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	Which interrupt are these tests for?
*
*		set		The data set to act on.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_parm_list_test(	void*					handle,
								U32						which,
								const set_get_list_t*	set)
{
	int				errs	= 0;
	unsigned long	get;
	int				i;
	U32				status;

	if (set->name)
		label_text(set->name);

	status	= hpdi32_init(handle);

	for (i = 0; i < (int) set->size; i++)
	{
		status	= hpdi32_config(handle,
								set->parm,
								which,
								set->list[i].set,
								&get);

		if ((status == GSC_SUCCESS) && (get != set->list[i].get))
			status	= GSC_FAILED;

	}

	hpdi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_callback_arg_tests
*
*	Purpose:
*
*		Perform Interrupt Callback Argument tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	The interrupt to test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_callback_arg_tests(void* handle, U32 which)
{
	static const set_get_t	list[]	=
	{
		/* set								get								*/
		{ 1024,								1024							},
		{ GSC_NO_CHANGE,					1024							},
		{ GSC_DEFAULT,						HPDI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ 0,								0								},
		{ GSC_NO_CHANGE,					0								},
		{ GSC_DEFAULT,						HPDI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ 2048,								2048							},
		{ GSC_NO_CHANGE,					2048							},
		{ GSC_DEFAULT,						HPDI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ HPDI32_IRQ_CALLBACK_ARG_DEFAULT,	HPDI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ GSC_NO_CHANGE,					HPDI32_IRQ_CALLBACK_ARG_DEFAULT	},
		{ GSC_DEFAULT,						HPDI32_IRQ_CALLBACK_ARG_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Callback Argument",
		/* parm	*/	HPDI32_IRQ_CALLBACK_ARG,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs;

	errs	= _irq_parm_list_test(handle, which, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_callback_func_tests
*
*	Purpose:
*
*		Perform Interrupt Callback Function tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	The interrupt to test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static void _callback(U32 arg1, U32 arg2, U32 arg3) {}	/*lint !e715 */

static int _irq_callback_func_tests(void* handle, U32 which)
{
	static set_get_t	list[9];

	static const set_get_list_t	set	=
	{
		/* name	*/	"Callback Function",
		/* parm	*/	HPDI32_IRQ_CALLBACK_ARG,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs;

	list[0].set	= (unsigned long) _callback;
	list[0].get	= (unsigned long) _callback;
	list[1].set	= GSC_NO_CHANGE;
	list[1].get	= (unsigned long) _callback;
	list[2].set	= GSC_DEFAULT;
	list[2].get	= HPDI32_IRQ_CALLBACK_FUNC_DEFAULT;
	list[3].set	= 0;
	list[3].get	= 0;
	list[4].set	= GSC_NO_CHANGE;
	list[4].get	= 0;
	list[5].set	= GSC_DEFAULT;
	list[5].get	= HPDI32_IRQ_CALLBACK_FUNC_DEFAULT;
	list[6].set	= HPDI32_IRQ_CALLBACK_FUNC_DEFAULT;
	list[6].get	= HPDI32_IRQ_CALLBACK_FUNC_DEFAULT;
	list[7].set	= GSC_NO_CHANGE;
	list[7].get	= HPDI32_IRQ_CALLBACK_FUNC_DEFAULT;
	list[8].set	= GSC_DEFAULT;
	list[8].get	= HPDI32_IRQ_CALLBACK_FUNC_DEFAULT;

	errs	= _irq_parm_list_test(handle, which, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_enable_tests
*
*	Purpose:
*
*		Perform Interrupt Enable tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	The interrupt to test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_enable_tests(void* handle, U32 which)
{
	static const set_get_t	list[]	=
	{
		/* set							get							*/
		{ HPDI32_IRQ_ENABLE_NO,			HPDI32_IRQ_ENABLE_NO		},
		{ GSC_NO_CHANGE,				HPDI32_IRQ_ENABLE_NO		},
		{ GSC_DEFAULT,					HPDI32_IRQ_ENABLE_DEFAULT	},
		{ HPDI32_IRQ_ENABLE_YES,		HPDI32_IRQ_ENABLE_YES		},
		{ GSC_NO_CHANGE,				HPDI32_IRQ_ENABLE_YES		},
		{ GSC_DEFAULT,					HPDI32_IRQ_ENABLE_DEFAULT	},
		{ HPDI32_IRQ_ENABLE_DEFAULT,	HPDI32_IRQ_ENABLE_DEFAULT	},
		{ GSC_NO_CHANGE,				HPDI32_IRQ_ENABLE_DEFAULT	},
		{ GSC_DEFAULT,					HPDI32_IRQ_ENABLE_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"Enable",
		/* parm	*/	HPDI32_IRQ_ENABLE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs;

	errs	= _irq_parm_list_test(handle, which, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_state_tests
*
*	Purpose:
*
*		Perform Interrupt State tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	The interrupt to test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_state_tests(void* handle, U32 which)
{
	static const set_get_t	list[]	=
	{
		/* set				get							*/
		{ 0,				HPDI32_IRQ_STATE_INACTIVE	},
		{ GSC_NO_CHANGE,	HPDI32_IRQ_STATE_INACTIVE	},
		{ GSC_DEFAULT,		HPDI32_IRQ_STATE_INACTIVE	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	"State",
		/* parm	*/	HPDI32_IRQ_STATE,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int	errs;

	errs	= _irq_parm_list_test(handle, which, &set);
	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_trigger_config_tests
*
*	Purpose:
*
*		Perform Interrupt Trigger Configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	The interrupt to test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_trigger_config_tests(void* handle, U32 which)
{
	static const set_get_t	list[]	=
	{
		/* set									get									*/
		{ HPDI32_IRQ_TRIGGER_CONFIG_EDGE_LOW,	HPDI32_IRQ_TRIGGER_CONFIG_EDGE_LOW	},
		{ GSC_NO_CHANGE,						HPDI32_IRQ_TRIGGER_CONFIG_EDGE_LOW	},
		{ GSC_DEFAULT,							HPDI32_IRQ_TRIGGER_CONFIG_DEFAULT	},
		{ HPDI32_IRQ_TRIGGER_CONFIG_EDGE_HI,	HPDI32_IRQ_TRIGGER_CONFIG_EDGE_HI	},
		{ GSC_NO_CHANGE,						HPDI32_IRQ_TRIGGER_CONFIG_EDGE_HI	},
		{ GSC_DEFAULT,							HPDI32_IRQ_TRIGGER_CONFIG_DEFAULT	},
		{ HPDI32_IRQ_TRIGGER_CONFIG_LEVEL_LOW,	HPDI32_IRQ_TRIGGER_CONFIG_LEVEL_LOW	},
		{ GSC_NO_CHANGE,						HPDI32_IRQ_TRIGGER_CONFIG_LEVEL_LOW	},
		{ GSC_DEFAULT,							HPDI32_IRQ_TRIGGER_CONFIG_DEFAULT	},
		{ HPDI32_IRQ_TRIGGER_CONFIG_LEVEL_HI,	HPDI32_IRQ_TRIGGER_CONFIG_LEVEL_HI	},
		{ GSC_NO_CHANGE,						HPDI32_IRQ_TRIGGER_CONFIG_LEVEL_HI	},
		{ GSC_DEFAULT,							HPDI32_IRQ_TRIGGER_CONFIG_DEFAULT	},
		{ HPDI32_IRQ_TRIGGER_CONFIG_DEFAULT,	HPDI32_IRQ_TRIGGER_CONFIG_DEFAULT	},
		{ GSC_NO_CHANGE,						HPDI32_IRQ_TRIGGER_CONFIG_DEFAULT	},
		{ GSC_DEFAULT,							HPDI32_IRQ_TRIGGER_CONFIG_DEFAULT	}
	};

	static const set_get_list_t	set	=
	{
		/* name	*/	NULL,
		/* parm	*/	HPDI32_IRQ_TRIGGER_CONFIG,
		/* list	*/	list,
		/* size	*/	ARRAY_ELEMENTS(list)
	};

	int				errs;
	unsigned long	feature;
	U32				status;

	label_text("Trigger Config");

	status	= HPDI32_MISC_FEATURES__ICR(handle, &feature);

	if (status != GSC_SUCCESS)
	{
		errs	= 1;
		printf("FAIL <---\n");
	}
	else if (feature != HPDI32_MISC_FEATURES_PRESENT)
	{
		errs	= 0;
		printf("SKIPPED (feature unavailable)\n");
	}
	else
	{
		errs	= _irq_parm_list_test(handle, which, &set);
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_config_tests
*
*	Purpose:
*
*		Perform Interrupt configuration tests for the given interrupt.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		which	The interrupt to test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_config_tests(void* handle, U32 which)
{
	int			errs	= 0;
	const char*	name;

	name	= _get_irq_name(which);
	label_text(name);
	printf("\n");

	label_level_inc();
	errs	+= _irq_callback_arg_tests(handle, which);
	errs	+= _irq_callback_func_tests(handle, which);
	errs	+= _irq_enable_tests(handle, which);
	errs	+= _irq_state_tests(handle, which);
	errs	+= _irq_trigger_config_tests(handle, which);
	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	irq_config_tests
*
*	Purpose:
*
*		Perform Interrupt configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int irq_config_tests(void* handle)
{
	int	errs	= 0;
	U32	which;

	label_text("Interrupt Configuration");
	printf("\n");

	label_level_inc();

	for (which = 0x1; which & HPDI32_WHICH_IRQ_ALL; which *= 2)
		errs	+= _irq_config_tests(handle, which);

	label_level_dec();
	return(errs);
}



/*****************************************************************************
*
*	Function:	_irq_callback_test
*
*	Purpose:
*
*		Perform Interrupt Notification testing using the Callback feature.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_callback_test(void* handle)
{
	label_text("Callback");
	printf("SKIPPED\n");
	return(0);
}



/*****************************************************************************
*
*	Function:	_irq_wait_test
*
*	Purpose:
*
*		Perform Interrupt Notification testing using the Wait feature.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

static int _irq_wait_test(void* handle)
{
	U32			buf[20];
	int			errs;
	fifo_info_t	info;
	U32			size;
	U32			status;
	U32			xfer;

	label_text("Wait");
	_irq_callback_arg2	= 0;
	memset(buf, 0, sizeof(buf));
	status	= hpdi32_init(handle);
	fifo_info_tx(&info, &size);

	if (status == GSC_SUCCESS)
		status	= HPDI32_IO_MODE__TX_PIO(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_TX_AUTO_START__NO(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_TX_ENABLE__YES(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_TX_CLOCK_DIVIDER__SET(handle, 0x4000);

	if (status == GSC_SUCCESS)
		status	= HPDI32_IRQ_TRIGGER_CONFIG__TX_E_LEV_HI(handle);

	if (status == GSC_SUCCESS)
		status	= HPDI32_IRQ_ENABLE__TX_E_YES(handle);

	if ((info != FIFO_NONE) && (status == GSC_SUCCESS))
		status	= hpdi32_write(handle, buf, sizeof(buf), &xfer);

	if ((info != FIFO_NONE) && (status == GSC_SUCCESS))
	{
		status	= hpdi32_irq_wait(handle, HPDI32_WHICH_IRQ_TX_E, 1000);

		if (status == GSC_WAIT_TIMEOUT)
			status	= GSC_SUCCESS;
	}

	hpdi32_init(handle);
	errs	= (status == GSC_SUCCESS) ? 0 : 1;

	if (info == FIFO_NONE)
	{
		errs	= 0;
		printf("SKIPPED (no Tx FIFOs)\n");
	}
	else if (errs)
	{
		printf("FAIL <---\n");
	}
	else
	{
		printf("PASS\n");
	}

	return(errs);
}



/*****************************************************************************
*
*	Function:	irq_notify_tests
*
*	Purpose:
*
*		Perform Interrupt notification tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
*****************************************************************************/

int irq_notify_tests(void* handle)
{
	int	errs	= 0;

	label_text("Interrupt Notification");
	printf("\n");
	label_level_inc();

	errs	+= _irq_callback_test(handle);
	errs	+= _irq_wait_test(handle);

	label_level_dec();
	return(errs);
}


