// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/utils/util_file.c $
// $Rev: 23665 $
// $Date: 2013-10-31 16:20:26 -0500 (Thu, 31 Oct 2013) $

#include "hpdi32_utils.h"

//#include <ctype.h>
//#include <stdio.h>
#include <string.h>



/******************************************************************************
*
*	Function:	file_close
*
*	Purpose:
*
*		Perform a close on an open file.
*
*	Arguments:
*
*		name	The name of the open file.
*
*		file	An open file handle.
*
*	Returned:
*
*		else	The operation failed.
*		NULL	The operation succeeded.
*
******************************************************************************/

FILE* file_close(const char* name, FILE* file)
{
	int		i;
	FILE*	ret;

	if (name == NULL)
	{
		ret	= (void*) 1;
		printf("ERROR: NULL argument: 'name'\n");
	}
	else if (file == NULL)
	{
		ret	= (void*) 1;
		printf("ERROR: NULL argument: 'file'\n");
	}
	else
	{
		i	= fclose(file);

		if (i)
		{
			ret	= (void*) 1;
			printf("ERROR: unable to close '%s'\n", name);
		}
		else
		{
			ret	= NULL;
		}
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_create
*
*	Purpose:
*
*		Create a file with the given name. The file is opened in binary write
*		mode.
*
*	Arguments:
*
*		name	The name of the file to create.
*
*	Returned:
*
*		NULL	The operation failed.
*		else	A valid file handle.
*
******************************************************************************/

FILE* file_create(const char* name)
{
	FILE*	ret;

	if (name == NULL)
	{
		ret	= NULL;
		printf("ERROR: NULL argument: 'name'\n");
	}
	else if (name[0] == 0)
	{
		ret	= NULL;
		printf("ERROR: invalid argument: 'name': '%s'\n", name);
	}
	else
	{
		ret	= fopen(name, "w+b");

		if (ret == NULL)
			printf("ERROR: file create failed: '%s'\n", name);
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_eof
*
*	Purpose:
*
*		Check to see if we're at the end of the file.
*
*	Arguments:
*
*		name	The name of the open file.
*
*		file	An open file handle.
*
*		eof		Set to 1 if we're at the end and 0 if not.
*
*	Returned:
*
*		0		All went well.
*		-1		There was a problem.
*
******************************************************************************/

int file_eof(const char* name, FILE* file, int* eof)
{
	int	ret;

	if (name == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'name'\n");
	}
	else if (file == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'file'\n");
	}
	else if (eof == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'eof'\n");
	}
	else
	{
		ret		= 0;
		eof[0]	= feof(file);
		eof[0]	= eof[0] ? 1 : 0;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_open
*
*	Purpose:
*
*		Perform an open given a file name. The file is opened in binary read
*		mode.
*
*	Arguments:
*
*		name	The name of the file to open.
*
*	Returned:
*
*		NULL	The operation failed.
*		else	A valid file handle.
*
******************************************************************************/

FILE* file_open(const char* name)
{
	FILE*	ret;

	if (name == NULL)
	{
		ret	= NULL;
		printf("ERROR: NULL argument: 'name'\n");
	}
	else if (name[0] == 0)
	{
		ret	= NULL;
		printf("ERROR: invalid argument: 'name': '%s'\n", name);
	}
	else
	{
		ret	= fopen(name, "rb");

		if (ret == NULL)
			printf("ERROR: unable to open '%s'\n", name);
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_read
*
*	Purpose:
*
*		Read data from an open file.
*
*	Arguments:
*
*		name	The name of the open file.
*
*		file	An open file handle.
*
*		buf		Put the data here.
*
*		size	The number of bytes to read.
*
*	Returned:
*
*		>= 0	The number of bytes read.
*		-1		There was a problem.
*
******************************************************************************/

long file_read(const char* name, FILE* file, char* buf, size_t size)
{
	int		eof;
	long	ret;

	if (name == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'name'\n");
	}
	else if (file == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'file'\n");
	}
	else if (buf == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'buf'\n");
	}
	else if (size < 1)
	{
		ret	= -1;
		printf("ERROR: invalid argument: 'size': %ld\n", (long) size);
	}
	else
	{
		memset(buf, 0, size);
		ret	= fread(buf, 1, size, file);
		eof	= feof(file);

		if (ret == size)
		{
		}
		else if (eof)
		{
		}
		else
		{
			ret	= -1;
			printf("ERROR: read failure of %ld bytes on '%s'\n", (long) size, name);
		}
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_read_line
*
*	Purpose:
*
*		Read a line of text from an open file.
*
*	Arguments:
*
*		name	The name of the open file.
*
*		file	An open file handle.
*
*		dst		Put the line of text here. The line may be terminated in DOS
*				or UNIX format. The returned line is always null terminated.
*				The EOL characters are also removed.
*
*		size	The size of the buffer.
*
*	Returned:
*
*		>= 0	The number of bytes read.
*		-1		There was a problem.
*
******************************************************************************/

long file_read_line(const char* name, FILE* file, char* dst, size_t size)
{
	char	buf[4096];
	int		eof;
	int		i;
	int		len;
	long	pos;
	char*	psz;
	long	qty;
	int		ret;

	for (;;)	// A convenience loop.
	{
		if (name == NULL)
		{
			ret	= -1;
			printf("ERROR: NULL argument: 'name'\n");
			break;
		}

		if (file == NULL)
		{
			ret	= -1;
			printf("ERROR: NULL argument: 'file'\n");
			break;
		}

		if (dst == NULL)
		{
			ret	= -1;
			printf("ERROR: NULL argument: 'dst'\n");
			break;
		}

		if (size < 1)
		{
			ret	= -1;
			printf("ERROR: invalid argument: 'size': %ld\n", (long) size);
			break;
		}

		pos	= file_tell(name, file);

		if (pos < 0)
		{
			ret	= -1;
			break;
		}

		memset(dst, 0, size);
		qty	= file_read(name, file, buf, sizeof(buf));

		if (qty == 0)
		{
			i	= file_eof(name, file, &eof);

			if (i == -1)
				ret	= -1;
			else if (eof)
				ret	= 0;
			else
				ret	= -1;

			dst[0]	= 0;
			break;
		}

		buf[sizeof(buf) - 1]	= 0;
		psz	= strchr(buf, '\n');

		if (psz)
			psz[1]	= 0;

		psz	= strchr(buf, '\r');

		if (psz == NULL)
			;
		else if (psz[1] != '\n')
			psz[1]	= 0;

		len	= strlen(buf);
		i	= file_seek(name, file, pos + len);

		if (i)
		{
			ret	= -1;
		}
		else
		{
			ret	= len;

			if (len >= (int) size)
			{
				strncpy(dst, buf, size);
				dst[size - 1]	= 0;
			}
			else
			{
				strncpy(dst, buf, len);
				dst[len]	= 0;
			}

			psz	= strchr(dst, '\n');

			if (psz)
				psz[0]	= 0;

			psz	= strchr(dst, '\r');

			if (psz)
				psz[0]	= 0;
		}

		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_seek
*
*	Purpose:
*
*		Report the current position in a file.
*
*	Arguments:
*
*		name	The name of the open file.
*
*		file	An open file handle.
*
*		pos		The absolute position to seek to.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

int file_seek(const char* name, FILE* file, long pos)
{
	int	ret;
	int	i;

	if (name == NULL)
	{
		ret	= 1;
		printf("ERROR: NULL argument: 'name'\n");
	}
	else if (file == NULL)
	{
		ret	= 1;
		printf("ERROR: NULL argument: 'file'\n");
	}
	else if (pos < 0)
	{
		ret	= 1;
		printf("ERROR: invalid argument: 'pos': %ld\n", pos);
	}
	else
	{
		i	= fseek(file, pos, SEEK_SET);

		if (i)
		{
			ret	= 1;
			printf("ERROR: fseek(): file access error: %s\n", name);
		}
		else
		{
			ret	= 0;
		}
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_tell
*
*	Purpose:
*
*		Report the current position in a file.
*
*	Arguments:
*
*		name	The name of the open file.
*
*		file	An open file handle.
*
*	Returned:
*
*		>= 0	The current location.
*		-1		There was a problem.
*
******************************************************************************/

long file_tell(const char* name, FILE* file)
{
	long	ret;

	if (name == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'name'\n");
	}
	else if (file == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'file'\n");
	}
	else
	{
		ret	= ftell(file);

		if (ret == -1)
			printf("ERROR: ftell(): file access error: %s\n", name);
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_size
*
*	Purpose:
*
*		Report the size of the file.
*
*	Arguments:
*
*		name	The name of the open file.
*
*		file	An open file handle.
*
*	Returned:
*
*		>= 0	The file size.
*		-1		There was a problem.
*
******************************************************************************/

long file_size(const char* name, FILE* file)
{
	int		i;
	long	pos;
	long	ret;

	for (;;)	// A convenience loop.
	{
		if (name == NULL)
		{
			ret	= -1;
			printf("ERROR: NULL argument: 'name'\n");
			break;
		}

		if (file == NULL)
		{
			ret	= -1;
			printf("ERROR: NULL argument: 'file'\n");
			break;
		}

		// Obtain the starting position so we can restore it later.
		pos	= ftell(file);

		if (pos == -1)
		{
			ret	= -1;
			printf("ERROR: ftell(): file access error: %s\n", name);
			break;
		}

		// Advance to the end of the file.
		i	= fseek(file, 0, SEEK_END);

		if (i)
		{
			ret	= -1;
			printf("ERROR: fseek(): file access error: %s\n", name);
			fseek(file, pos, SEEK_SET);
			break;
		}

		// Retrieve the current position, which is the file size.
		ret	= ftell(file);

		if (ret == -1)
		{
			printf("ERROR: ftell(): file access error: %s\n", name);
			break;
		}

		// Restore the starting position.
		i	= fseek(file, pos, SEEK_SET);

		if (i)
		{
			ret	= -1;
			printf("ERROR: fseek(): file access error: %s\n", name);
			break;
		}

		break;
	}

	return(ret);
}



/******************************************************************************
*
*	Function:	file_write
*
*	Purpose:
*
*		Write data to an open file.
*
*	Arguments:
*
*		name	The name of the open file.
*
*		file	An open file handle.
*
*		buf		The source data is here.
*
*		size	The number of bytes to write.
*
*	Returned:
*
*		>= 0	The number of bytes written.
*		-1		There was a problem.
*
******************************************************************************/

long file_write(const char* name, FILE* file, const char* src, size_t size)
{
	long	ret;

	if (name == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'name'\n");
	}
	else if (file == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'file'\n");
	}
	else if (src == NULL)
	{
		ret	= -1;
		printf("ERROR: NULL argument: 'src'\n");
	}
	else if (size < 1)
	{
		ret	= -1;
		printf("ERROR: invalid argument: 'size': %ld\n", (long) size);
	}
	else
	{
		ret	= fwrite(src, 1, size, file);

		if (ret == -1)
		{
			printf("ERROR: write failure of %ld bytes on '%s'\n", (long) size, name);
			ret	= -1;
		}
	}

	return(ret);
}


